package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Building;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class BuildingsCommandsHandler {

    private final RequestConfig config;

    public BuildingsCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.buildings}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Building[] buildings = (Building[]) EntitiesRequests.get(config,
                    RequestConfig.BUILDING_ENTITIES_QUERY,
                    Building.class).getDeserializedEntities();
            EntityExporter.export(buildings, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Buildings successfully queried!");
    }

    @CommandHandler({"{query.buildings}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Building[] buildings = (Building[]) EntitiesRequests.get(config,
                    RequestConfig.BUILDING_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Building.class).getDeserializedEntities();
            EntityExporter.export(buildings, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Building with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.buildings}", "{httpMethod.get}", "{query.sub.byName}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Building[] buildings = (Building[]) EntitiesRequests.get(config,
                    RequestConfig.BUILDING_ENTITIES_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Building.class).getDeserializedEntities();
            EntityExporter.export(buildings, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Building of name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.buildings}", "{httpMethod.get}", "{query.sub.byRoom}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByRoomID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Building[] buildings = (Building[]) EntitiesRequests.get(config,
                    RequestConfig.BUILDING_ENTITIES_QUERY + RequestConfig.ROOM_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Building.class).getDeserializedEntities();
            EntityExporter.export(buildings, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Building with room id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.buildings}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Building[] buildings = (Building[]) EntityExporter.importToObjects(fileType, filePath, Building.class);
            EntitiesRequests.post(config, RequestConfig.BUILDING_ENTITIES_QUERY, buildings);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Building/Buildings successfully created!%n");
    }

    @CommandHandler({"{query.buildings}", "{httpMethod.put}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePutByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Building[] buildings = (Building[]) EntityExporter.importToObjects(fileType, filePath, Building.class);
            EntitiesRequests.put(config, RequestConfig.BUILDING_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    buildings[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Building with id: %d successfully modified!%n", id);
    }

    @CommandHandler({"{query.buildings}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.BUILDING_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Building with id: %d successfully deleted!%n", id);
    }
}
