package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.SchoolClass;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Subject;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class ClassesCommandsHandler {

    private final RequestConfig config;

    public ClassesCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Classes successfully queried!");
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.byStudent}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByStudentEntityID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.STUDENT_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with student id: %s successfully queried!%n", id);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.byStudent}",
            "{query.sub.byName}", "<java.lang.String>", "{query.sub.bySurname}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByStudentEntityFirstAndLastName(@NotNull String name, @NotNull String lastName,
                                                   @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.STUDENT_QUERY + RequestConfig.NAME_QUERY + "/"
                            + name + "/" + lastName,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with student name: %s surname: %s successfully queried!%n", name, lastName);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.byTeacher}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByTeacherID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.TEACHER_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with teacher id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.byTeacher}",
            "{query.sub.byName}", "<java.lang.String>", "{query.sub.bySurname}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByTeacherFirstAndLastName(@NotNull String firstName, @NotNull String lastName,
                                                   @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.TEACHER_QUERY + RequestConfig.NAME_QUERY + "/"
                            + firstName + "/" + lastName,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with teacher name: %s surname: %s successfully queried!%n", firstName, lastName);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.byRoom}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByRoomID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.ROOM_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with room id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.byRoom}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByRoomName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.ROOM_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with room name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.bySubject}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.SUBJECT_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with subject id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.get}", "{query.sub.bySubject}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classes = (SchoolClass[]) EntitiesRequests.get(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.SUBJECT_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    SchoolClass.class).getDeserializedEntities();
            EntityExporter.export(classes, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with subject name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classEntities = (SchoolClass[]) EntityExporter.importToObjects(fileType, filePath, SchoolClass.class);
            EntitiesRequests.post(config, RequestConfig.CLASSES_ENTITIES_QUERY, classEntities);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class/Classes successfully created!%n");
    }

    @CommandHandler({"{query.classes}", "{httpMethod.put}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePutByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolClass[] classEntities = (SchoolClass[]) EntityExporter.importToObjects(fileType, filePath, SchoolClass.class);
            EntitiesRequests.put(config, RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    classEntities[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with id: %d successfully modified!%n", id);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.put}", "<java.lang.Integer>", "{query.sub.bySubject}",
            "{query.sub.doAdd}", "[query.sub.type]", "<java.lang.String>"})
    public void handlePutAddSubjectByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] classEntities = (Subject []) EntityExporter.importToObjects(fileType, filePath,
                    Subject.class);
            EntitiesRequests.put(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id +
                            RequestConfig.SUBJECT_QUERY + RequestConfig.ADD_QUERY, classEntities[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with id: %d successfully added subject!%n", id);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.put}", "<java.lang.Integer>", "{query.sub.bySubject}",
            "{query.sub.doRemove}", "[query.sub.type]", "<java.lang.String>"})
    public void handlePutRemoveSubjectByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] classEntities = (Subject []) EntityExporter.importToObjects(fileType, filePath,
                    Subject.class);
            EntitiesRequests.put(config,
                    RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id +
                            RequestConfig.SUBJECT_QUERY + RequestConfig.REMOVE_QUERY, classEntities[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with id: %d successfully removed subject!%n", id);
    }

    @CommandHandler({"{query.classes}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.CLASSES_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Class with id: %d successfully deleted!%n", id);
    }
}
