package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Room;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class RoomsCommandsHandler {

    private final RequestConfig config;

    public RoomsCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Rooms successfully queried!");
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}", "{query.sub.byName}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}", "{query.sub.byCapacity}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByCapacity(Integer capacity, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.CAPACITY_QUERY + "/" + capacity,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with capacity: %d successfully queried!%n", capacity);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}", "{query.sub.byCapacity}", "<java.lang.Integer>",
            "<java.lang.Integer>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByCapacityInterval(Integer min, Integer max, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.CAPACITY_QUERY + "/between/" + min + "/" + max,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Building with capacity from interval: <%d;%d> successfully queried!%n", min, max);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}", "{query.sub.byBuilding}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByBuildingsID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.BUILDING_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with building id: %s successfully queried!%n", id);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}", "{query.sub.byBuilding}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByBuildingsName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.BUILDING_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with building name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}", "{query.sub.byClass}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByClassID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.CLASS_QUERY + RequestConfig.ID_QUERY +
                    "/" + id,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with class id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.get}", "{query.sub.byClass}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByClassName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntitiesRequests.get(config,
                    RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.CLASS_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Room.class).getDeserializedEntities();
            EntityExporter.export(rooms, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with class name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntityExporter.importToObjects(fileType, filePath, Room.class);
            EntitiesRequests.post(config, RequestConfig.ROOMS_ENTITIES_QUERY, rooms);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room/Rooms successfully created!%n");
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.put}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePutByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Room[] rooms = (Room[]) EntityExporter.importToObjects(fileType, filePath, Room.class);
            EntitiesRequests.put(config, RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    rooms[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with id: %d successfully modified!%n", id);
    }

    @CommandHandler({"{query.rooms}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.ROOMS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Room with id: %d successfully deleted!%n", id);
    }
}
