package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.ScheduleActionDate;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class ScheduleActionsDatesCommandsHandler {

    private final RequestConfig config;

    public ScheduleActionsDatesCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            ScheduleActionDate[] scheduleActionDates = (ScheduleActionDate[]) EntitiesRequests.get(config,
                    RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY,
                    ScheduleActionDate.class).getDeserializedEntities();
            EntityExporter.export(scheduleActionDates, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Schedule action dates successfully queried!");
    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            ScheduleActionDate[] scheduleActionDates = (ScheduleActionDate[]) EntitiesRequests.get(config,
                    RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    ScheduleActionDate.class).getDeserializedEntities();
            EntityExporter.export(scheduleActionDates, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Schedule action date with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.get}", "{query.sub.bySubject}",
            "<java.lang.Integer>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            ScheduleActionDate[] scheduleActionDates = (ScheduleActionDate[]) EntitiesRequests.get(config,
                    RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY + RequestConfig.SUBJECT_QUERY + RequestConfig.ID_QUERY +
                            "/" + id,
                    ScheduleActionDate.class).getDeserializedEntities();
            EntityExporter.export(scheduleActionDates, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Schedule action date with subject id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.get}", "{query.sub.bySubstitutions}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetSubstitutions(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            ScheduleActionDate[] scheduleActionDates = (ScheduleActionDate[]) EntitiesRequests.get(config,
                    RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY + RequestConfig.SUBSTITUTION_QUERY,
                    ScheduleActionDate.class).getDeserializedEntities();
            EntityExporter.export(scheduleActionDates, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Schedule action date with substitutions successfully queried!%n");
    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.get}", "{query.sub.bySubstitutions}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetSubstitutionsByDate(@NotNull String date, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            ScheduleActionDate[] scheduleActionDates = (ScheduleActionDate[]) EntitiesRequests.get(config,
                    RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY + RequestConfig.SUBSTITUTION_QUERY +
                            RequestConfig.DATE_QUERY + "/" + date,
                    ScheduleActionDate.class).getDeserializedEntities();
            EntityExporter.export(scheduleActionDates, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Schedule action date of date: %s successfully queried!%n",
                date);
    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.get}", "{query.sub.bySubstitutions}",
            "<java.lang.String>", "{query.sub.byTeacher}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetSubstitutionsByDateAndTeacherID(@NotNull String date, Integer id,
                                                         @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            ScheduleActionDate[] scheduleActionDates = (ScheduleActionDate[]) EntitiesRequests.get(config,
                    RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY + RequestConfig.SUBSTITUTION_QUERY +
                            RequestConfig.DATE_QUERY + "/" + date + RequestConfig.TEACHER_QUERY + RequestConfig.ID_QUERY +
                            "/" + id,
                    ScheduleActionDate.class).getDeserializedEntities();
            EntityExporter.export(scheduleActionDates, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Schedule action date of date: %s with teacher id: %d successfully queried!%n",
                date, id);
    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.get}", "{query.sub.bySubstitutions}",
            "<java.lang.String>", "{query.sub.byStudent}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetSubstitutionsByDateAndStudentEntityID(@NotNull String date, Integer id,
                                                               @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            ScheduleActionDate[] scheduleActionDates = (ScheduleActionDate[]) EntitiesRequests.get(config,
                    RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY + RequestConfig.SUBSTITUTION_QUERY +
                            RequestConfig.DATE_QUERY + "/" + date + RequestConfig.STUDENT_QUERY + RequestConfig.ID_QUERY +
                            "/" + id,
                    ScheduleActionDate.class).getDeserializedEntities();
            EntityExporter.export(scheduleActionDates, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Schedule action date of date: %s with student id: %d successfully queried!%n",
                date, id);
    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            ScheduleActionDate[] scheduleActionDates = (ScheduleActionDate[]) EntityExporter.importToObjects(fileType, filePath, ScheduleActionDate.class);
            EntitiesRequests.post(config, RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY, scheduleActionDates);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Schedule action date/Schedule action dates successfully created!%n");

    }

    @CommandHandler({"{query.scheduleActionsDates}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Schedule action date with id: %d successfully deleted!%n", id);
    }
}
