package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.SchoolHour;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class SchoolHoursCommandsHandler {

    private final RequestConfig config;

    public SchoolHoursCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.schoolHours}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolHour[] schoolHours = (SchoolHour[]) EntitiesRequests.get(config,
                    RequestConfig.SCHOOLHOURS_ENTITIES_QUERY,
                    SchoolHour.class).getDeserializedEntities();
            EntityExporter.export(schoolHours, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("School hour successfully queried!");
    }

    @CommandHandler({"{query.schoolHours}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolHour[] schoolHours = (SchoolHour[]) EntitiesRequests.get(config,
                    RequestConfig.SCHOOLHOURS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    SchoolHour.class).getDeserializedEntities();
            EntityExporter.export(schoolHours, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("School hour with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.schoolHours}", "{httpMethod.get}", "{query.sub.byName}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolHour[] schoolHours = (SchoolHour[]) EntitiesRequests.get(config,
                    RequestConfig.SCHOOLHOURS_ENTITIES_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    SchoolHour.class).getDeserializedEntities();
            EntityExporter.export(schoolHours, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("School hour with name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.schoolHours}", "{httpMethod.get}", "{query.sub.byScheduleAction}",
            "<java.lang.Integer>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByScheduleActionID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolHour[] schoolHours = (SchoolHour[]) EntitiesRequests.get(config,
                    RequestConfig.SCHOOLHOURS_ENTITIES_QUERY + RequestConfig.SCHEDULEACTION_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    SchoolHour.class).getDeserializedEntities();
            EntityExporter.export(schoolHours, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("School hour with schedule action id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.schoolHours}", "{httpMethod.get}", "{query.sub.byScheduleActionDate}",
            "<java.lang.Integer>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByScheduleActionDateEntityID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolHour[] schoolHours = (SchoolHour[]) EntitiesRequests.get(config,
                    RequestConfig.SCHOOLHOURS_ENTITIES_QUERY + RequestConfig.SCHEDULEACTIONDATE_QUERY +
                            RequestConfig.ID_QUERY + "/" + id,
                    SchoolHour.class).getDeserializedEntities();
            EntityExporter.export(schoolHours, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("School hour with schedule action date id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.schoolHours}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolHour[] schoolHours = (SchoolHour[]) EntityExporter.importToObjects(fileType, filePath, SchoolHour.class);
            EntitiesRequests.post(config, RequestConfig.SCHOOLHOURS_ENTITIES_QUERY, schoolHours);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("School hour/School hours successfully created!");
    }

    @CommandHandler({"{query.schoolHours}", "{httpMethod.put}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePutByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SchoolHour[] schoolHours = (SchoolHour[]) EntityExporter.importToObjects(fileType, filePath, SchoolHour.class);
            EntitiesRequests.put(config, RequestConfig.SCHOOLHOURS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    schoolHours[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("School hour with id: %d successfully modified!%n", id);
    }

    @CommandHandler({"{query.schoolHours}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.SCHOOLHOURS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("School hour with id: %d successfully deleted!%n", id);
    }
}
