package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Student;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Subject;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class StudentsCommandsHandler {

    private final RequestConfig config;

    public StudentsCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.students}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTS_ENTITIES_QUERY,
                    Student.class).getDeserializedEntities();
            EntityExporter.export(students, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Students successfully queried!");
    }

    @CommandHandler({"{query.students}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Student.class).getDeserializedEntities();
            EntityExporter.export(students, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.students}", "{httpMethod.get}", "{query.sub.byName}", "<java.lang.String>",
            "{query.sub.bySurname}", "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByFirstAndLastName(@NotNull String firstName, @NotNull String lastName,
                                            @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.NAME_QUERY + "/" + firstName + "/" + lastName,
                    Student.class).getDeserializedEntities();
            EntityExporter.export(students, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with name: %s surname: %s successfully queried!%n", firstName, lastName);
    }

    @CommandHandler({"{query.students}", "{httpMethod.get}", "{query.sub.byClass}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByClassID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.CLASS_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Student.class).getDeserializedEntities();
            EntityExporter.export(students, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with subject id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.students}", "{httpMethod.get}", "{query.sub.byClass}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByClassName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.CLASS_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Student.class).getDeserializedEntities();
            EntityExporter.export(students, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with class name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.students}", "{httpMethod.get}", "{query.sub.bySubject}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.SUBJECT_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Student.class).getDeserializedEntities();
            EntityExporter.export(students, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with subject id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.students}", "{httpMethod.get}", "{query.sub.bySubject}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.SUBJECT_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Student.class).getDeserializedEntities();
            EntityExporter.export(students, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with subject name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.students}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntityExporter.importToObjects(fileType, filePath, Student.class);
            EntitiesRequests.post(config, RequestConfig.STUDENTS_ENTITIES_QUERY, students);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student/Students successfully created!%n");
    }

    @CommandHandler({"{query.students}", "{httpMethod.put}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePutByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Student[] students = (Student[]) EntityExporter.importToObjects(fileType, filePath, Student.class);
            EntitiesRequests.put(config, RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    students[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with id: %d successfully modified!%n", id);
    }

    @CommandHandler({"{query.students}", "{httpMethod.put}", "<java.lang.Integer>", "{query.sub.bySubject}",
            "{query.sub.doAdd}", "[query.sub.type]", "<java.lang.String>"})
    public void handlePutAddSubjectByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntityExporter.importToObjects(fileType, filePath, Subject.class);
            EntitiesRequests.put(config, RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id
                    + RequestConfig.SUBJECT_QUERY + RequestConfig.ADD_QUERY, subjects[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with id: %d successfully added subject!%n", id);
    }

    @CommandHandler({"{query.students}", "{httpMethod.put}", "<java.lang.Integer>", "{query.sub.bySubject}",
            "{query.sub.doRemove}", "[query.sub.type]", "<java.lang.String>"})
    public void handlePutRemoveSubjectByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntityExporter.importToObjects(fileType, filePath, Subject.class);
            EntitiesRequests.put(config, RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id
                    + RequestConfig.SUBJECT_QUERY + RequestConfig.REMOVE_QUERY, subjects[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with id: %d successfully removed subject!%n", id);
    }

    @CommandHandler({"{query.students}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.STUDENTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student with id: %d successfully deleted!%n", id);
    }
}
