package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.StudentsSubject;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class StudentsSubjectsCommandsHandler {

    private final RequestConfig config;

    public StudentsSubjectsCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.studentsSubjects}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            StudentsSubject[] studentsSubjects = (StudentsSubject[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTSSUBJECTS_ENTITIES_QUERY,
                    StudentsSubject.class).getDeserializedEntities();
            EntityExporter.export(studentsSubjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Students subjects successfully queried!");
    }

    @CommandHandler({"{query.studentsSubjects}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            StudentsSubject[] studentsSubjects = (StudentsSubject[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTSSUBJECTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    StudentsSubject.class).getDeserializedEntities();
            EntityExporter.export(studentsSubjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student subject with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.studentsSubjects}", "{httpMethod.get}", "{query.sub.byStudent}", "<java.lang.Integer>",
            "{query.sub.bySubject}", "<java.lang.Integer>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByStudentEntityAndSubjectID(Integer studentId, Integer subjectId,
                                                     @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            StudentsSubject[] studentsSubjects = (StudentsSubject[]) EntitiesRequests.get(config,
                    RequestConfig.STUDENTSSUBJECTS_ENTITIES_QUERY + RequestConfig.STUDENT_QUERY + RequestConfig.ID_QUERY +
                            "/" + studentId + RequestConfig.SUBJECT_QUERY + RequestConfig.ID_QUERY + "/" + subjectId,
                    StudentsSubject.class).getDeserializedEntities();
            EntityExporter.export(studentsSubjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Student subject with student id: %d subject id: %d successfully queried!%n", studentId, subjectId);
    }
}
