package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Subject;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class SubjectsCommandsHandler {

    private final RequestConfig config;

    public SubjectsCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Subjects successfully queried!");
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.byName}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.bySubjectList}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectListID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.SUBJECTLIST_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with subject list id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.bySubjectList}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectListName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.SUBJECTLIST_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with subject list name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.byClass}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByClassID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.CLASS_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with class id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.byClass}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByClassName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.CLASS_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with class name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.byScheduleAction}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByScheduleActionID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.SCHEDULEACTION_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with schedule action id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.byTeacher}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByTeacherID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.TEACHER_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with teacher id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.byTeacher}", "{query.sub.byName}",
            "<java.lang.String>", "{query.sub.bySurname}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByTeacherFirstAndLastName(@NotNull String firstName, @NotNull String lastName,
                                                   @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.TEACHER_QUERY + RequestConfig.NAME_QUERY +
                            "/" + firstName + "/" + lastName,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with teacher name: %s surname: %s successfully queried!%n", firstName, lastName);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.byStudent}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByStudentEntityID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.STUDENT_QUERY + RequestConfig.ID_QUERY +
                            "/" + id,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.get}", "{query.sub.byStudent}", "{query.sub.byName}",
            "<java.lang.String>", "{query.sub.bySurname}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByStudentEntityFirstAndLastName(@NotNull String firstName, @NotNull String lastName,
                                                         @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntitiesRequests.get(config,
                    RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.STUDENT_QUERY + RequestConfig.NAME_QUERY + "/" +
                            firstName + "/" + lastName,
                    Subject.class).getDeserializedEntities();
            EntityExporter.export(subjects, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with student name: %s surname: %s successfully queried!%n", firstName, lastName);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntityExporter.importToObjects(fileType, filePath, Subject.class);
            EntitiesRequests.post(config, RequestConfig.SUBJECTS_ENTITIES_QUERY, subjects);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject/Subjects successfully created!%n");
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.put}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePutByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Subject[] subjects = (Subject[]) EntityExporter.importToObjects(fileType, filePath, Subject.class);
            EntitiesRequests.put(config, RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    subjects[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with id: %d successfully modified!%n", id);
    }

    @CommandHandler({"{query.subjects}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.SUBJECTS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Subject with id: %d successfully deleted!%n", id);
    }
}
