package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.SubjectList;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Teacher;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class TeachersCommandsHandler {

    private final RequestConfig config;

    public TeachersCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntitiesRequests.get(config,
                    RequestConfig.TEACHERS_ENTITIES_QUERY,
                    Teacher.class).getDeserializedEntities();
            EntityExporter.export(teachers, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Teachers successfully queried!");
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntitiesRequests.get(config,
                    RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Teacher.class).getDeserializedEntities();
            EntityExporter.export(teachers, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.get}", "{query.sub.byName}", "<java.lang.String>",
            "{query.sub.bySurname}", "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByFirstAndLastName(@NotNull String firstName, @NotNull String lastName,
                                            @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntitiesRequests.get(config,
                    RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.NAME_QUERY + "/" + firstName + "/" + lastName,
                    Teacher.class).getDeserializedEntities();
            EntityExporter.export(teachers, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with name: %s surname: %s successfully queried!%n", firstName, lastName);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.get}", "{query.sub.bySubjectList}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectListID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntitiesRequests.get(config,
                    RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.SUBJECTLIST_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Teacher.class).getDeserializedEntities();
            EntityExporter.export(teachers, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with subject list id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.get}", "{query.sub.bySubjectList}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectListName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntitiesRequests.get(config,
                    RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.SUBJECTLIST_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Teacher.class).getDeserializedEntities();
            EntityExporter.export(teachers, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with subject list name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.get}", "{query.sub.bySubject}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntitiesRequests.get(config,
                    RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.SUBJECT_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Teacher.class).getDeserializedEntities();
            EntityExporter.export(teachers, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with subject id: %d successfully queried!%n", id);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.get}", "{query.sub.bySubject}", "{query.sub.byName}",
            "<java.lang.String>", "[query.sub.type]", "<java.lang.String>"})
    public void handleGetBySubjectName(@NotNull String name, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntitiesRequests.get(config,
                    RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.SUBJECT_QUERY + RequestConfig.NAME_QUERY + "/" + name,
                    Teacher.class).getDeserializedEntities();
            EntityExporter.export(teachers, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with subject name: %s successfully queried!%n", name);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntityExporter.importToObjects(fileType, filePath, Teacher.class);
            EntitiesRequests.post(config, RequestConfig.TEACHERS_ENTITIES_QUERY, teachers);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher/Teachers successfully created!");
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.put}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePutByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Teacher[] teachers = (Teacher[]) EntityExporter.importToObjects(fileType, filePath, Teacher.class);
            EntitiesRequests.put(config, RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    teachers[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with id: %d successfully modified!%n", id);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.put}", "<java.lang.Integer>", "{query.sub.bySubjectList}",
            "{query.sub.doAdd}", "[query.sub.type]", "<java.lang.String>"})
    public void handlePutAddSubjectListByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SubjectList[] teachers = (SubjectList[]) EntityExporter.importToObjects(fileType, filePath, SubjectList.class);
            EntitiesRequests.put(config, RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id
                    + RequestConfig.SUBJECTLIST_QUERY + RequestConfig.ADD_QUERY, teachers[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with id: %d successfully added subject list!%n", id);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.put}", "<java.lang.Integer>", "{query.sub.bySubjectList}",
            "{query.sub.doRemove}", "[query.sub.type]", "<java.lang.String>"})
    public void handlePutRemoveSubjectListByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            SubjectList[] teachers = (SubjectList[]) EntityExporter.importToObjects(fileType, filePath, SubjectList.class);
            EntitiesRequests.put(config, RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id
                    + RequestConfig.SUBJECTLIST_QUERY + RequestConfig.REMOVE_QUERY, teachers[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with id: %d successfully removed subject list!%n", id);
    }

    @CommandHandler({"{query.teachers}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.TEACHERS_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Teacher with id: %d successfully deleted!%n", id);
    }
}
