package cz.zcu.fav.kiv.mjakubas.saf.config.configs;

import cz.zcu.fav.kiv.mjakubas.saf.io.properties.PropertiesIO;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.util.Properties;

/**
 * Represents a net config of application.
 */
public class NetConfig {

    public final String address;
    public final String port;
    public final String username;
    public final String password;

    private final Authenticator authenticator;

    private NetConfig(@NotNull Properties netProperty) throws IOException {
        this.address = PropertiesIO.getValue(netProperty, "address");
        this.port = PropertiesIO.getValue(netProperty, "port");
        this.username = PropertiesIO.getValue(netProperty, "username");
        this.password = PropertiesIO.getValue(netProperty, "password");

        this.authenticator = new Authenticator() {
            @Override
            protected PasswordAuthentication getPasswordAuthentication() {
                return new PasswordAuthentication(username, password.toCharArray());
            }
        };
    }

    /**
     * Creates new instance of {@link NetConfig} from net config '.properties'
     * file.
     *
     * @param netProperty net config properties file
     * @return new {@link NetConfig}
     * @throws IOException if any properties expected value is missing
     */
    public static NetConfig loadNetConfig(@NotNull Properties netProperty) throws IOException {
        return new NetConfig(netProperty);
    }

    // get

    /**
     * Gets net config connection {@link Authenticator}.
     *
     * @return net config connection {@link Authenticator}
     */
    public Authenticator getAuthenticator() {
        return authenticator;
    }

    public String getAddress() {
        return address;
    }

    public String getPort() {
        return port;
    }

    public String getUsername() {
        return username;
    }

    public String getPassword() {
        return password;
    }
}
