package cz.zcu.fav.kiv.mjakubas.saf.entity;

import com.fasterxml.jackson.annotation.JsonInclude;

/**
 * Represents a School Class entity implementation for frontend of school agenda.
 */
@JsonInclude(JsonInclude.Include.NON_DEFAULT)
public final class SchoolClass extends ASimpleEntity {

    private int idClasses;
    private String className;

    private Room room;
    private Teacher teacher;
    private Student[] students;
    private Subject[] subjects;

    // constructors

    /**
     * Instantiates new empty School Class entity.
     */
    public SchoolClass() {
    }

    /**
     * Instantiates new School Class entity with all its fields.
     *
     * @param idClasses school class id
     * @param className school class name
     * @param room      classroom
     * @param teacher   school class teacher
     * @param students  school class students
     * @param subjects  school class subjects
     */
    public SchoolClass(int idClasses, String className, Room room, Teacher teacher, Student[] students, Subject[] subjects) {
        this.idClasses = idClasses;
        this.className = className;
        this.room = room;
        this.teacher = teacher;
        this.students = students;
        this.subjects = subjects;
    }

    // override

    @Override
    public String toString() {
        return String.format("[%d] %s", idClasses, className);
    }

    @Override
    public int getID() {
        return idClasses;
    }

    @Override
    public String getSimpleIdentifier() {

        return String.format("%s", className);
    }

    // get

    /**
     * Get class id.
     *
     * @return class id
     */
    public int getIdClasses() {
        return idClasses;
    }


    /**
     * Get class name.
     *
     * @return class name
     */
    public String getClassName() {
        return className;
    }

    /**
     * Get classroom.
     *
     * @return classroom
     */
    public Room getRoom() {
        return room;
    }

    /**
     * Get class teacher.
     *
     * @return class teacher
     */
    public Teacher getTeacher() {
        return teacher;
    }

    /**
     * Get class students.
     *
     * @return class students
     */
    public Student[] getStudents() {
        return students;
    }

    /**
     * Get class subjects.
     *
     * @return class subjects
     */
    public Subject[] getSubjects() {
        return subjects;
    }

    // set

    /**
     * Set class id.
     *
     * @param idClasses class id
     */
    public void setIdClasses(int idClasses) {
        this.idClasses = idClasses;
    }

    /**
     * Set class name.
     *
     * @param className class name
     */
    public void setClassName(String className) {
        this.className = className;
    }

    /**
     * Set classroom.
     *
     * @param roomEntities classroom
     */
    public void setRoom(Room roomEntities) {
        this.room = roomEntities;
    }

    /**
     * Set class teacher.
     *
     * @param teacherEntities class teacher
     */
    public void setTeacher(Teacher teacherEntities) {
        this.teacher = teacherEntities;
    }

    /**
     * Set class students.
     *
     * @param studentEntities class students
     */
    public void setStudents(Student[] studentEntities) {
        this.students = studentEntities;
    }

    /**
     * Set class subjects.
     *
     * @param subjectEntities class subjects
     */
    public void setSubjects(Subject[] subjectEntities) {
        this.subjects = subjectEntities;
    }
}
