package cz.zcu.fav.kiv.mjakubas.saf.entity;

import com.fasterxml.jackson.annotation.JsonInclude;

/**
 * Represents a School Hour entity implementation for frontend of school agenda.
 */
@JsonInclude(JsonInclude.Include.NON_DEFAULT)
public final class SchoolHour extends ASimpleEntity {

    private int idSchoolhours;
    private String schoolhourName;
    private String schoolhourBegin;
    private String schoolhourEnd;

    private ScheduleAction[] scheduleactions;
    private ScheduleActionDate[] scheduleActionDates;

    // constructors

    /**
     * Instantiates new empty School Hour entity.
     */
    public SchoolHour() {
    }

    /**
     * Instantiates new School Hour entity with all its fields.
     *
     * @param idSchoolhours              school hour id
     * @param schoolhourName             school hour name
     * @param schoolhourBegin            school hour begin time
     * @param schoolhourEnd              school hour end time
     * @param scheduleactions            school hour schedule action
     * @param scheduleActionDateEntities school hour schedule action date
     */
    public SchoolHour(int idSchoolhours, String schoolhourName, String schoolhourBegin, String schoolhourEnd, ScheduleAction[] scheduleactions, ScheduleActionDate[] scheduleActionDateEntities) {
        this.idSchoolhours = idSchoolhours;
        this.schoolhourName = schoolhourName;
        this.schoolhourBegin = schoolhourBegin;
        this.schoolhourEnd = schoolhourEnd;
        this.scheduleactions = scheduleactions;
        this.scheduleActionDates = scheduleActionDateEntities;
    }

    // override

    @Override
    public int getID() {
        return idSchoolhours;
    }

    @Override
    public String getSimpleIdentifier() {
        return schoolhourName;
    }

    @Override
    public String toString() {
        return getIdentifier();
    }

    // get

    /**
     * Get school hour id.
     *
     * @return school hour id
     */
    public int getIdSchoolhours() {
        return idSchoolhours;
    }

    /**
     * Get school hour name.
     *
     * @return school hour name
     */
    public String getSchoolhourName() {
        return schoolhourName;
    }

    /**
     * Get school hour begin time.
     *
     * @return school hour begin time
     */
    public String getSchoolhourBegin() {
        return schoolhourBegin;
    }

    /**
     * Get school hour end time.
     *
     * @return school hour end time
     */
    public String getSchoolhourEnd() {
        return schoolhourEnd;
    }

    /**
     * Get school hour schedule actions.
     *
     * @return school hour schedule actions
     */
    public ScheduleAction[] getScheduleactions() {
        return scheduleactions;
    }

    /**
     * Get school hour schedule action dates.
     *
     * @return school hour schedule action dates
     */
    public ScheduleActionDate[] getScheduleActionDates() {
        return scheduleActionDates;
    }

    // set

    /**
     * Set school hour id.
     *
     * @param idSchoolhours school hour id
     */
    public void setIdSchoolhours(int idSchoolhours) {
        this.idSchoolhours = idSchoolhours;
    }

    /**
     * Set school hour name.
     *
     * @param schoolhourName school hour name
     */
    public void setSchoolhourName(String schoolhourName) {
        this.schoolhourName = schoolhourName;
    }

    /**
     * Set school hour begin time.
     *
     * @param schoolhourBegin school hour begin time
     */
    public void setSchoolhourBegin(String schoolhourBegin) {
        this.schoolhourBegin = schoolhourBegin;
    }

    /**
     * Set school hour end time.
     *
     * @param schoolhourEnd school hour end time
     */
    public void setSchoolhourEnd(String schoolhourEnd) {
        this.schoolhourEnd = schoolhourEnd;
    }

    /**
     * Set school hour schedule action.
     *
     * @param scheduleactions school hour schedule action
     */
    public void setScheduleactions(ScheduleAction[] scheduleactions) {
        this.scheduleactions = scheduleactions;
    }

    /**
     * Set school hour schedule action date.
     *
     * @param scheduleActionDates school hour schedule action date
     */
    public void setScheduleActionDates(ScheduleActionDate[] scheduleActionDates) {
        this.scheduleActionDates = scheduleActionDates;
    }
}
