package cz.zcu.fav.kiv.mjakubas.saf.entity;

import com.fasterxml.jackson.annotation.JsonInclude;

/**
 * Represents a Students Subject entity implementation for frontend of school agenda.
 */
@JsonInclude(JsonInclude.Include.NON_DEFAULT)
public final class StudentsSubject extends ASimpleEntity {

    private int idStudentssubjects;

    private Evaluation[] evaluations;

    private Subject subject;
    private Student student;

    // constructors

    /**
     * Instantiates new empty Students Subject entity.
     */
    public StudentsSubject() {
    }

    /**
     * Instantiates new Students Subject entity with all its fields.
     *
     * @param idStudentssubjects students subjects id
     * @param evaluationEntities students subjects evaluations.
     */
    public StudentsSubject(int idStudentssubjects, Evaluation[] evaluationEntities) {
        this.idStudentssubjects = idStudentssubjects;
        this.evaluations = evaluationEntities;
    }

    // override

    @Override
    public int getID() {
        return idStudentssubjects;
    }

    @Override
    public String getSimpleIdentifier() {
        StringBuilder identifierBuilder = new StringBuilder();

        String teacherIdentifier =
                (student != null) ?
                        String.format(" {[%d] %s %s)}", student.getIdStudents(), student.getFirstName(),
                                student.getLastName()) : "";
        String scheduleActionDateIdentifier =
                (subject != null) ? String.format("{[%d] %s}", subject.getID(),
                        subject.getSubjectName()) : "";
        identifierBuilder.append(scheduleActionDateIdentifier).append(teacherIdentifier);

        return identifierBuilder.toString();
    }

    @Override
    public String toString() {
        return getIdentifier();
    }

    // get

    /**
     * Get students subject id.
     *
     * @return students subject id
     */
    public int getIdStudentssubjects() {
        return idStudentssubjects;
    }

    /**
     * Get students evaluation.
     *
     * @return students evaluation
     */
    public Evaluation[] getEvaluations() {
        return evaluations;
    }

    public Subject getSubject() {
        return subject;
    }

    public Student getStudent() {
        return student;
    }

    // set

    /**
     * Set students subject id.
     *
     * @param idStudentssubjects students subject id
     */
    public void setIdStudentssubjects(int idStudentssubjects) {
        this.idStudentssubjects = idStudentssubjects;
    }

    /**
     * Set students subjects evaluation.
     *
     * @param evaluations students subjects evaluation
     */
    public void setEvaluations(Evaluation[] evaluations) {
        this.evaluations = evaluations;
    }

    public void setSubject(Subject subject) {
        this.subject = subject;
    }

    public void setStudent(Student student) {
        this.student = student;
    }
}
