package cz.zcu.fav.kiv.mjakubas.saf.entity;

import com.fasterxml.jackson.annotation.JsonInclude;

/**
 * Represents a Subject entity implementation for frontend of school agenda.
 */
@JsonInclude(JsonInclude.Include.NON_DEFAULT)
public final class Subject extends ASimpleEntity {

    private int idSubjects;
    private String subjectName;

    private ScheduleAction[] scheduleactions;
    private StudentsSubject[] studentssubjects;

    private ScheduleActionDate[] scheduleactiondates;

    public ScheduleActionDate[] getScheduleactiondates() {
        return scheduleactiondates;
    }

    private SubjectList subjectlist;
    private Teacher teacher;

    // constructors

    /**
     * Instantiates new empty Subject entity.
     */
    public Subject() {
    }

    /**
     * Instantiates new Subject entity with all its fields.
     *
     * @param idSubjects       subjects id
     * @param subjectName      subjects name
     * @param scheduleactions  subjects schedule action
     * @param studentssubjects subjects students subject
     */
    public Subject(int idSubjects, String subjectName, ScheduleAction[] scheduleactions, StudentsSubject[] studentssubjects) {
        this.idSubjects = idSubjects;
        this.subjectName = subjectName;
        this.scheduleactions = scheduleactions;
        this.studentssubjects = studentssubjects;
    }

    // override

    @Override
    public int getID() {
        return idSubjects;
    }

    @Override
    public String getSimpleIdentifier() {
        StringBuilder identifierBuilder = new StringBuilder();

        String teacherIdentifier =
                (teacher != null) ?
                        String.format(" {[%d] %s %s)}", teacher.getIdTeachers(), teacher.getFirstName(),
                                teacher.getLastName()) : "";
        String scheduleActionDateIdentifier =
                (subjectlist != null) ? String.format("{[%d] %s}", subjectlist.getID(),
                        subjectlist.getSubjectName()) : "";
        identifierBuilder.append(subjectName).append(" ").append(scheduleActionDateIdentifier).append(teacherIdentifier);

        return identifierBuilder.toString();
    }

    @Override
    public String toString() {
        return getIdentifier();
    }

    // get

    /**
     * Get subjects teacher.
     *
     * @return subjects teacher
     */
    public Teacher getTeacher() {
        return teacher;
    }

    /**
     * Get subject subjects lists.
     *
     * @return subject subjects lists
     */
    public SubjectList getSubjectlist() {
        return subjectlist;
    }

    /**
     * Get subjects id.
     *
     * @return subjects id
     */
    public int getIdSubjects() {
        return idSubjects;
    }

    /**
     * Get subjects name.
     *
     * @return subjects name
     */
    public String getSubjectName() {
        return subjectName;
    }

    /**
     * Get subjects schedule action.
     *
     * @return subjects schedule action
     */
    public ScheduleAction[] getScheduleactions() {
        return scheduleactions;
    }

    /**
     * Get subjects students subjects.
     *
     * @return subjects students subjects
     */
    public StudentsSubject[] getStudentssubjects() {
        return studentssubjects;
    }

    // set

    /**
     * Set subjects subject list.
     *
     * @param subjectlist subjects list
     */
    public void setSubjectlist(SubjectList subjectlist) {
        this.subjectlist = subjectlist;
    }

    /**
     * Set subjects schedule action dates.
     *
     * @param scheduleactiondates subjects schedule action dates
     */
    public void setScheduleactiondates(ScheduleActionDate[] scheduleactiondates) {
        this.scheduleactiondates = scheduleactiondates;
    }

    /**
     * Set subjects id.
     *
     * @param idSubjects subjects id
     */
    public void setIdSubjects(int idSubjects) {
        this.idSubjects = idSubjects;
    }

    /**
     * Set subjects subject name.
     *
     * @param subjectName subjects name
     */
    public void setSubjectName(String subjectName) {
        this.subjectName = subjectName;
    }

    /**
     * Set subjects schedule action.
     *
     * @param scheduleactions subjects schedule action
     */
    public void setScheduleactions(ScheduleAction[] scheduleactions) {
        this.scheduleactions = scheduleactions;
    }

    /**
     * Set subjects students subjects.
     *
     * @param studentssubjects subjects students subjects
     */
    public void setStudentssubjects(StudentsSubject[] studentssubjects) {
        this.studentssubjects = studentssubjects;
    }

    /**
     * Set subjects teacher.
     *
     * @param teacher subjects teacher
     */
    public void setTeacher(Teacher teacher) {
        this.teacher = teacher;
    }
}
