package cz.zcu.fav.kiv.mjakubas.saf.entity;

import com.fasterxml.jackson.annotation.JsonInclude;

/**
 * Represents a Subject List entity implementation for frontend of school agenda.
 */
@JsonInclude(JsonInclude.Include.NON_DEFAULT)
public final class SubjectList extends ASimpleEntity {

    private int idSubjectlist;
    private String subjectName;

    private Subject[] subjects;

    // constructors

    /**
     * Instantiates new empty Subject List entity.
     */
    public SubjectList() {
    }

    /**
     * Instantiates new Subject List entity with all its fields.
     *
     * @param idSubjectlist subject list id
     * @param subjectName   subject list name
     * @param subjects      subject lists subjects
     */
    public SubjectList(int idSubjectlist, String subjectName, Subject[] subjects) {
        this.idSubjectlist = idSubjectlist;
        this.subjectName = subjectName;
        this.subjects = subjects;
    }

    // override

    @Override
    public int getID() {
        return idSubjectlist;
    }

    @Override
    public String getSimpleIdentifier() {
        return String.format("%s", subjectName);
    }

    @Override
    public String toString() {
        return getIdentifier();
    }

    // get

    /**
     * Get subject list id.
     *
     * @return subject list id
     */
    public int getIdSubjectlist() {
        return idSubjectlist;
    }

    /**
     * Get subject list name.
     *
     * @return subject list name
     */
    public String getSubjectName() {
        return subjectName;
    }

    /**
     * Get subject lists subjects.
     *
     * @return subject lists subjects
     */
    public Subject[] getSubjects() {
        return subjects;
    }

    // set

    /**
     * Set subject lists id.
     *
     * @param idSubjectlist subject lists id
     */
    public void setIdSubjectlist(int idSubjectlist) {
        this.idSubjectlist = idSubjectlist;
    }

    /**
     * Set subject lists name.
     *
     * @param subjectName subject lists name
     */
    public void setSubjectName(String subjectName) {
        this.subjectName = subjectName;
    }

    /**
     * Set subject lists subjects.
     *
     * @param subjects subject lists subjects
     */
    public void setSubjects(Subject[] subjects) {
        this.subjects = subjects;
    }
}
