package cz.zcu.fav.kiv.mjakubas.saf.gui;

import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.menu.EntityControlsMenu;
import cz.zcu.fav.kiv.mjakubas.saf.gui.menu.I18NMenu;
import cz.zcu.fav.kiv.mjakubas.saf.gui.view.*;
import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.control.MenuBar;
import javafx.scene.layout.*;
import javafx.stage.Stage;
import org.kordamp.bootstrapfx.BootstrapFX;

import java.util.Objects;

/**
 * Represents a school agenda front-end application in a GUI form.
 */
public class GUI extends Application {

    private static final int MIN_WIDTH = 1280;
    private static final int MIN_HEIGHT = 576;

    @Override
    public void start(Stage primaryStage) throws Exception {
        primaryStage.setMinWidth(MIN_WIDTH);
        primaryStage.setMinHeight(MIN_HEIGHT);

        Importer.getFileChooser().setParentStage(primaryStage);

        VBox primaryPane = new VBox();

        // :)

        primaryStage.titleProperty().bind(I18N.getLocalizationTool().createStringBinding("app.title"));
        buildPrimaryPane(primaryPane, primaryStage);

        createApplicationContent(primaryPane);

        // :(

        Scene primaryScene = new Scene(primaryPane);
        primaryScene.getStylesheets()
                .add(Objects.requireNonNull(GUI.class.getResource("stylesheet.css")).toExternalForm());
        primaryScene.getStylesheets().add(BootstrapFX.bootstrapFXStylesheet());

        primaryStage.setScene(primaryScene);
        primaryStage.show();
        primaryPane.requestFocus();
    }

    public void doLaunch() {
        launch();
    }

    // build

    /**
     * Builds application root {@link Pane}.
     *
     * @param primaryPane  application root {@link Pane}
     * @param primaryStage application root {@link Stage}
     */
    private void buildPrimaryPane(VBox primaryPane, Stage primaryStage) {
        primaryPane.prefHeightProperty().bind(primaryStage.heightProperty());
        primaryPane.getStyleClass().addAll("pane-primary");

        buildPrimaryMenuBar(primaryPane);
        buildContentPane(primaryPane);
    }

    private static final int[] COLUMN_WIDTHS = new int[]{20, 30, 50};

    /**
     * Builds content {@link Pane}.
     *
     * @param primaryPane root {@link Pane} of content {@link Pane}
     */
    private void buildContentPane(VBox primaryPane) {
        VBox col1 = createContentColumnPane(true);
        VBox col2 = createContentColumnPane(true);
        VBox col3 = createContentColumnPane(false);

        GridPane contentPane = new GridPane();
        contentPane.add(col1, 0, 0);
        contentPane.add(col2, 1, 0);
        contentPane.add(col3, 2, 0);

        ColumnConstraints c0 = new ColumnConstraints();
        c0.setPercentWidth(COLUMN_WIDTHS[0]);
        ColumnConstraints c1 = new ColumnConstraints();
        c1.setPercentWidth(COLUMN_WIDTHS[1]);
        ColumnConstraints c2 = new ColumnConstraints();
        c2.setPercentWidth(COLUMN_WIDTHS[2]);
        contentPane.getColumnConstraints().addAll(c0, c1, c2);

        RowConstraints r0 = new RowConstraints();
        r0.prefHeightProperty().bind(primaryPane.heightProperty());
        contentPane.getRowConstraints().add(r0);

        contentPane.prefWidthProperty().bind(primaryPane.widthProperty());
        contentPane.prefHeightProperty().bind(primaryPane.heightProperty());

        primaryPane.getChildren().add(contentPane);
    }

    /**
     * Builds new {@link MenuBar} used for application main menu.
     */
    private void buildPrimaryMenuBar(VBox primaryPane) {
        MenuBar primaryMenuBar = new MenuBar();
        primaryMenuBar.getStyleClass().add("menu-primary");

        primaryMenuBar.getMenus().addAll(
                new EntityControlsMenu(I18N.getLocalizationTool().createStringBinding("app.menus.entity")).getMenu(),
                new I18NMenu(I18N.getLocalizationTool().createStringBinding("app.menus.i18n")).getMenu());

        primaryPane.getChildren().add(primaryMenuBar);
    }

    // create

    /**
     * Creates content {@link Pane} column {@link Pane}.
     *
     * @param hasBorder if content column {@link Pane} has border.
     * @return content column {@link Pane}
     */
    private VBox createContentColumnPane(boolean hasBorder) {
        VBox contentColumnPane = new VBox();

        if (hasBorder)
            contentColumnPane.getStyleClass()
                    .add("pane-border");

        return contentColumnPane;
    }

    /**
     * Fills content {@link Pane}.
     *
     * @param primaryPane application root {@link Pane}
     */
    private void createApplicationContent(VBox primaryPane) {
        EntityFormsPane formsPane = new EntityFormsPane();
        EntitiesSearchableButtonPane entitiesPane = new EntitiesSearchableButtonPane();
        CategorySearchableButtonPane categoryPane =
                new CategorySearchableButtonPane(EntitiesController.getEntitiesController(), entitiesPane, formsPane);
        entitiesPane.setReloadFunction(categoryPane);
        ViewContainer.getContainer().setEntitiesSearchableButtonPane(entitiesPane);

        GridPane contentPane = (GridPane) primaryPane.getChildren().get(1);
        VBox col0 = (VBox) contentPane.getChildren().get(0);
        VBox col1 = (VBox) contentPane.getChildren().get(1);
        VBox col2 = (VBox) contentPane.getChildren().get(2);

        col0.getChildren().add(categoryPane.getPane());
        col1.getChildren().add(entitiesPane.getPane());
        col2.getChildren().add(formsPane.getFormsPane());
    }
}
