package cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity;

import cz.zcu.fav.kiv.mjakubas.saf.entity.*;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.entities.*;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform.*;

/**
 * Contains all {@link AEntityController} implementations of all entities of school agenda.
 */
public class EntitiesController {
    private final AbsenceEntityController absenceEntityController;
    private final BuildingEntityController buildingEntityController;
    private final ClassEntityController classEntityController;
    private final EvaluationEntityController evaluationEntityController;
    private final RoomEntityController roomEntityController;
    private final ScheduleActionDateEntityController scheduleActionDateEntityController;
    private final ScheduleActionEntityController scheduleActionEntityController;
    private final SchoolDayEntityController schoolDayEntityController;
    private final SchoolHourEntityController schoolHourEntityController;
    private final StudentEntityController studentEntityController;
    private final StudentsSubjectEntityController studentsSubjectEntityController;
    private final SubjectEntityController subjectEntityController;
    private final SubjectListEntityController subjectListEntityController;
    private final TeacherEntityController teacherEntityController;

    private static final EntitiesController ENTITIES_CONTROLLER = new EntitiesController();

    public static EntitiesController getEntitiesController() {
        return ENTITIES_CONTROLLER;
    }

    // constructor

    /**
     * Instantiates new {@link EntitiesController}.
     */
    public EntitiesController() {
        absenceEntityController =
                new AbsenceEntityController(new AbsenceForm(), this);
        buildingEntityController =
                new BuildingEntityController(new BuildingForm(), this);
        classEntityController =
                new ClassEntityController(new ClassForm(), this);
        evaluationEntityController =
                new EvaluationEntityController(new EvaluationForm(), this);
        roomEntityController =
                new RoomEntityController(new RoomForm(), this);
        scheduleActionDateEntityController =
                new ScheduleActionDateEntityController(new ScheduleActionDateForm(), this);
        scheduleActionEntityController =
                new ScheduleActionEntityController(new ScheduleActionForm(), this);
        schoolDayEntityController =
                new SchoolDayEntityController(new SchoolDayForm(), this);
        schoolHourEntityController =
                new SchoolHourEntityController(new SchoolHourForm(), this);
        studentEntityController =
                new StudentEntityController(new StudentForm(), this);
        studentsSubjectEntityController =
                new StudentsSubjectEntityController(new StudentSubjectForm(), this);
        subjectEntityController =
                new SubjectEntityController(new SubjectForm(), this);
        subjectListEntityController =
                new SubjectListEntityController(new SubjectListForm(), this);
        teacherEntityController =
                new TeacherEntityController(new TeacherForm(), this);
    }

    // get

    /**
     * Gets {@link AEntityController} implementation for {@link Absence}.
     *
     * @return {@link AEntityController} implementation for {@link Absence}
     */
    public AbsenceEntityController getAbsenceEntityController() {
        return absenceEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link Building}.
     *
     * @return {@link AEntityController} implementation for {@link Building}
     */
    public BuildingEntityController getBuildingEntityController() {
        return buildingEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link SchoolClass}.
     *
     * @return {@link AEntityController} implementation for {@link SchoolClass}
     */
    public ClassEntityController getClassEntityController() {
        return classEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link Evaluation}.
     *
     * @return {@link AEntityController} implementation for {@link Evaluation}
     */
    public EvaluationEntityController getEvaluationEntityController() {
        return evaluationEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link Room}.
     *
     * @return {@link AEntityController} implementation for {@link Room}
     */
    public RoomEntityController getRoomEntityController() {
        return roomEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link ScheduleActionDate}.
     *
     * @return {@link AEntityController} implementation for {@link ScheduleActionDate}
     */
    public ScheduleActionDateEntityController getScheduleActionDateEntityController() {
        return scheduleActionDateEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link ScheduleAction}.
     *
     * @return {@link AEntityController} implementation for {@link ScheduleAction}
     */
    public ScheduleActionEntityController getScheduleActionEntityController() {
        return scheduleActionEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link SchoolDay}.
     *
     * @return {@link AEntityController} implementation for {@link SchoolDay}
     */
    public SchoolDayEntityController getSchoolDayEntityController() {
        return schoolDayEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link SchoolHour}.
     *
     * @return {@link AEntityController} implementation for {@link SchoolHour}
     */
    public SchoolHourEntityController getSchoolHourEntityController() {
        return schoolHourEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link Student}.
     *
     * @return {@link AEntityController} implementation for {@link Student}
     */
    public StudentEntityController getStudentEntityController() {
        return studentEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link StudentsSubject}.
     *
     * @return {@link AEntityController} implementation for {@link StudentsSubject}
     */
    public StudentsSubjectEntityController getStudentsSubjectEntityController() {
        return studentsSubjectEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link Subject}.
     *
     * @return {@link AEntityController} implementation for {@link Subject}
     */
    public SubjectEntityController getSubjectEntityController() {
        return subjectEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link SubjectList}.
     *
     * @return {@link AEntityController} implementation for {@link SubjectList}
     */
    public SubjectListEntityController getSubjectListEntityController() {
        return subjectListEntityController;
    }

    /**
     * Gets {@link AEntityController} implementation for {@link Teacher}.
     *
     * @return {@link AEntityController} implementation for {@link Teacher}
     */
    public TeacherEntityController getTeacherEntityController() {
        return teacherEntityController;
    }

    public void purgeFetchedData() {
        absenceEntityController.clearFetchedEntities();
        buildingEntityController.clearFetchedEntities();
        classEntityController.clearFetchedEntities();
        evaluationEntityController.clearFetchedEntities();
        roomEntityController.clearFetchedEntities();
        scheduleActionDateEntityController.clearFetchedEntities();
        scheduleActionEntityController.clearFetchedEntities();
        schoolDayEntityController.clearFetchedEntities();
        schoolHourEntityController.clearFetchedEntities();
        studentEntityController.clearFetchedEntities();
        studentsSubjectEntityController.clearFetchedEntities();
        subjectEntityController.clearFetchedEntities();
        subjectListEntityController.clearFetchedEntities();
        teacherEntityController.clearFetchedEntities();
    }
}
