package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform;

import com.dlsc.formsfx.model.structure.Field;
import com.dlsc.formsfx.model.structure.Form;
import com.dlsc.formsfx.model.structure.Group;
import com.dlsc.formsfx.model.structure.SingleSelectionField;
import com.dlsc.formsfx.view.renderer.FormRenderer;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Room;
import cz.zcu.fav.kiv.mjakubas.saf.entity.SchoolClass;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Subject;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Teacher;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.AEntityController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.ClassModel;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.IModel;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Represent all possible forms of {@link SchoolClass} entity.
 */
public class ClassForm extends AForm<SchoolClass> {

    @Override
    protected AEntityController<?> getEntityController() {
        return EntitiesController.getEntitiesController().getClassEntityController();
    }

    @Override
    protected Form createPostForm(IModel<SchoolClass> entityModel) {
        ClassModel model = (ClassModel) entityModel;
        SingleSelectionField<Room> allRoomsField =
                Field.ofSingleSelectionType(model.allRoomsProperty())
                        .label("form.label.rooms-all")
                        .required("form.required");
        model.selectedRoomProperty().bind(allRoomsField.selectionProperty());
        SingleSelectionField<Teacher> allTeachersField =
                Field.ofSingleSelectionType(model.allTeacherProperty())
                        .label("form.label.teachers-all")
                        .required("form.required");
        model.selectedTeacherProperty().bind(allTeachersField.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofStringType(model.classNameProperty())
                                .label("form.label.class-name")
                                .required("form.required")
                                .placeholder("form.placeholder.class-name"),
                        allRoomsField,
                        allTeachersField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createPostPane(Form postForm) {
        return new FormRenderer(postForm);
    }

    @Override
    protected Form createEditablePutForm(IModel<SchoolClass> entityModel) {
        ClassModel model = (ClassModel) entityModel;
        SingleSelectionField<Room> allRoomsField =
                Field.ofSingleSelectionType(model.allRoomsProperty(),
                                model.allRoomsProperty().indexOf(model.selectedRoomProperty().getValue()))
                        .label("form.label.rooms-all")
                        .required("form.required");
        model.selectedRoomProperty().bind(allRoomsField.selectionProperty());
        SingleSelectionField<Teacher> allTeachersField =
                Field.ofSingleSelectionType(model.allTeacherProperty(),
                                model.allTeacherProperty().indexOf(model.selectedTeacherProperty().getValue()))
                        .label("form.label.teachers-all")
                        .required("form.required");
        model.selectedTeacherProperty().bind(allTeachersField.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofIntegerType(model.classIDProperty())
                                .label("form.label.id")
                                .editable(false),
                        Field.ofStringType(model.classNameProperty())
                                .label("form.label.class-name")
                                .required("form.required")
                                .placeholder("form.placeholder.class-name"),
                        allRoomsField,
                        allTeachersField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createEditablePutPane(Form editablePutForm) {
        return new FormRenderer(editablePutForm);
    }

    @Override
    protected Form createAdditionalPutForm(IModel<?> entityModel) {
        ClassModel model = (ClassModel) entityModel;
        SingleSelectionField<Subject> allSubjectListsField =
                Field.ofSingleSelectionType(model.allSubjectsProperty())
                        .label("form.label.subjects-all")
                        .required("form.required");
        model.selectedSubjectsProperty().bind(allSubjectListsField.selectionProperty());

        return Form.of(
                Group.of(
                        allSubjectListsField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createNonEditablePutPane(Form nonEditablePutForm) {
        return new FormRenderer(nonEditablePutForm);
    }

    @Override
    protected Form createAdditionalDeletePutForm(IModel<?> entityModel) {
        ClassModel model = (ClassModel) entityModel;
        SingleSelectionField<Subject> allSubjectListsField =
                Field.ofSingleSelectionType(model.allTeacherSubjectsProperty())
                        .label("form.label.subjects-all")
                        .required("form.required");
        model.selectedSubjectsProperty().bind(allSubjectListsField.selectionProperty());

        return Form.of(
                Group.of(
                        allSubjectListsField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createNonEditableDeletePutPane(Form nonEditablePutForm) {
        return new FormRenderer(nonEditablePutForm);
    }

    @Override
    protected IModel<SchoolClass> createEmptyModel() {
        return new ClassModel();
    }

    @Override
    protected IModel<SchoolClass> createEntityModel(@NotNull SchoolClass entity) {
        IModel<SchoolClass> schoolClassModel = new ClassModel();
        schoolClassModel.importEntity(entity);
        return schoolClassModel;
    }

    @Override
    protected IModel<?> createAdditionalModel() {
        return null;
    }

    @Override
    protected Object getAdditionalEntity(IModel<SchoolClass> additionalModel) {
        ClassModel model = (ClassModel) additionalModel;
        return model.selectedSubjectsProperty().getValue();
    }
}
