package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform;

import com.dlsc.formsfx.model.structure.Field;
import com.dlsc.formsfx.model.structure.Form;
import com.dlsc.formsfx.model.structure.Group;
import com.dlsc.formsfx.model.structure.SingleSelectionField;
import com.dlsc.formsfx.model.validators.DoubleRangeValidator;
import com.dlsc.formsfx.model.validators.IntegerRangeValidator;
import com.dlsc.formsfx.view.renderer.FormRenderer;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Evaluation;
import cz.zcu.fav.kiv.mjakubas.saf.entity.StudentsSubject;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.AEntityController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.EvaluationModel;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.IModel;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Represent all possible forms of {@link Evaluation} entity.
 */
public class EvaluationForm extends AForm<Evaluation> {

    @Override
    protected AEntityController<?> getEntityController() {
        return EntitiesController.getEntitiesController().getEvaluationEntityController();
    }

    @Override
    protected Form createPostForm(IModel<Evaluation> entityModel) {
        EvaluationModel model = (EvaluationModel) entityModel;
        SingleSelectionField<StudentsSubject> allStudentsSubjects =
                Field.ofSingleSelectionType(model.allStudentsSubjectsProperty())
                        .label("form.label.students-subjects-all")
                        .required("form.required");
        model.selectedStudentsSubjectProperty().bind(allStudentsSubjects.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofStringType(model.testNameProperty())
                                .label("form.label.test-name")
                                .required("form.required")
                                .placeholder("form.placeholder.test-name"),
                        Field.ofDoubleType(model.gradeProperty())
                                .label("form.label.grade")
                                .required("form.required")
                                .placeholder("form.placeholder.grade")
                                .validate(DoubleRangeValidator.atLeast(1, "form.validator.positive-value")),
                        Field.ofIntegerType(model.weightProperty())
                                .label("form.label.weight")
                                .required("form.required")
                                .placeholder("form.placeholder.weight")
                                .validate(IntegerRangeValidator.atLeast(1, "form.validator.positive-value")),
                        allStudentsSubjects
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createPostPane(Form postForm) {
        return new FormRenderer(postForm);
    }

    @Override
    protected Form createEditablePutForm(IModel<Evaluation> entityModel) {
        EvaluationModel model = (EvaluationModel) entityModel;
        SingleSelectionField<StudentsSubject> allStudentsSubjects =
                Field.ofSingleSelectionType(model.allStudentsSubjectsProperty(),
                                model.allStudentsSubjectsProperty().indexOf(model.selectedStudentsSubjectProperty().getValue()))
                        .label("form.label.students-subjects-all")
                        .required("form.required");
        model.selectedStudentsSubjectProperty().bind(allStudentsSubjects.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofIntegerType(model.evaluationIDProperty())
                                .label("form.label.id")
                                .editable(false),
                        Field.ofStringType(model.testNameProperty())
                                .label("form.label.test-name")
                                .required("form.required")
                                .placeholder("form.placeholder.test-name"),
                        Field.ofDoubleType(model.gradeProperty())
                                .label("form.label.grade")
                                .required("form.required")
                                .placeholder("form.placeholder.grade")
                                .validate(DoubleRangeValidator.atLeast(1, "form.validator.positive-value")),
                        Field.ofIntegerType(model.weightProperty())
                                .label("form.label.weight")
                                .required("form.required")
                                .placeholder("form.placeholder.weight")
                                .validate(IntegerRangeValidator.atLeast(1, "form.validator.positive-value")),
                        allStudentsSubjects
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createEditablePutPane(Form editablePutForm) {
        return new FormRenderer(editablePutForm);
    }

    @Override
    protected Form createAdditionalPutForm(IModel<?> entityModel) {
        return null;
    }


    @Override
    protected Pane createNonEditablePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected Form createAdditionalDeletePutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditableDeletePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected IModel<Evaluation> createEmptyModel() {
        return new EvaluationModel();
    }

    @Override
    protected IModel<Evaluation> createEntityModel(@NotNull Evaluation entity) {
        IModel<Evaluation> evaluationModel = new EvaluationModel();
        evaluationModel.importEntity(entity);
        return evaluationModel;
    }

    @Override
    protected IModel<?> createAdditionalModel() {
        return null;
    }

    @Override
    protected Object getAdditionalEntity(IModel<Evaluation> additionalModel) {
        return null;
    }
}
