package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform;

import com.dlsc.formsfx.model.structure.Field;
import com.dlsc.formsfx.model.structure.Form;
import com.dlsc.formsfx.model.structure.Group;
import com.dlsc.formsfx.model.structure.SingleSelectionField;
import com.dlsc.formsfx.model.validators.IntegerRangeValidator;
import com.dlsc.formsfx.view.renderer.FormRenderer;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Building;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Room;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.AEntityController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.IModel;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.RoomModel;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Represent all possible forms of {@link Room} entity.
 */
public class RoomForm extends AForm<Room> {

    @Override
    protected AEntityController<?> getEntityController() {
        return EntitiesController.getEntitiesController().getRoomEntityController();
    }

    @Override
    protected Form createPostForm(IModel<Room> entityModel) {
        RoomModel model = (RoomModel) entityModel;
        SingleSelectionField<Building> allBuildings = Field.ofSingleSelectionType(model.allBuildingsProperty())
                .label("form.label.buildings-all")
                .required("form.required");
        model.selectedBuildingProperty().bind(allBuildings.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofStringType(model.roomNameProperty())
                                .label("form.label.room-name")
                                .required("form.required")
                                .placeholder("form.placeholder.room-name"),
                        Field.ofIntegerType(model.capacityProperty())
                                .label("form.label.capacity")
                                .required("form.required")
                                .placeholder("form.placeholder.capacity")
                                .validate(IntegerRangeValidator.atLeast(1, "form.validator.positive-value")),
                        allBuildings
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createPostPane(Form postForm) {
        return new FormRenderer(postForm);
    }

    @Override
    protected Form createEditablePutForm(IModel<Room> entityModel) {
        RoomModel model = (RoomModel) entityModel;
        SingleSelectionField<Building> allBuildings = Field.ofSingleSelectionType(model.allBuildingsProperty(),
                        model.allBuildingsProperty().indexOf(model.selectedBuildingProperty().getValue()))
                .label("form.label.buildings-all")
                .required("form.required");
        model.selectedBuildingProperty().bind(allBuildings.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofIntegerType(model.roomIDProperty())
                                .label("form.label.id")
                                .editable(false),
                        Field.ofStringType(model.roomNameProperty())
                                .label("form.label.room-name")
                                .required("form.required")
                                .placeholder("form.placeholder.room-name"),
                        Field.ofIntegerType(model.capacityProperty())
                                .label("form.label.capacity")
                                .required("form.required")
                                .placeholder("form.placeholder.capacity")
                                .validate(IntegerRangeValidator.atLeast(1, "form.validator.positive-value")),
                        allBuildings
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createEditablePutPane(Form editablePutForm) {
        return new FormRenderer(editablePutForm);
    }

    @Override
    protected Form createAdditionalPutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditablePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected Form createAdditionalDeletePutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditableDeletePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected IModel<Room> createEmptyModel() {
        return new RoomModel();
    }

    @Override
    protected IModel<Room> createEntityModel(@NotNull Room entity) {
        IModel<Room> roomModel = new RoomModel();
        roomModel.importEntity(entity);
        return roomModel;
    }

    @Override
    protected IModel<?> createAdditionalModel() {
        return null;
    }

    @Override
    protected Object getAdditionalEntity(IModel<Room> additionalModel) {
        return null;
    }

    // get
}
