package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform;

import com.dlsc.formsfx.model.structure.Field;
import com.dlsc.formsfx.model.structure.Form;
import com.dlsc.formsfx.model.structure.Group;
import com.dlsc.formsfx.model.structure.SingleSelectionField;
import com.dlsc.formsfx.view.renderer.FormRenderer;
import cz.zcu.fav.kiv.mjakubas.saf.entity.ScheduleAction;
import cz.zcu.fav.kiv.mjakubas.saf.entity.ScheduleActionDate;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.AEntityController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.IModel;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.ScheduleActionDateModel;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Represent all possible forms of {@link ScheduleActionDate} entity.
 */
public class ScheduleActionDateForm extends AForm<ScheduleActionDate> {
    @Override
    protected AEntityController<?> getEntityController() {
        return EntitiesController.getEntitiesController().getScheduleActionDateEntityController();
    }

    @Override
    protected Form createPostForm(IModel<ScheduleActionDate> entityModel) {
        ScheduleActionDateModel model = (ScheduleActionDateModel) entityModel;
        SingleSelectionField<ScheduleAction> allScheduleActions =
                Field.ofSingleSelectionType(model.allScheduleActionsProperty())
                        .label("form.label.scheduleaction-all")
                        .required("form.required");
        model.selectedScheduleActionProperty().bind(allScheduleActions.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofDate(model.dateProperty())
                                .label("form.label.date")
                                .required("form.required")
                                .placeholder("form.placeholder.new-date"),
                        Field.ofStringType(model.themeProperty())
                                .label("form.label.theme")
                                .required("form.required")
                                .placeholder("form.placeholder.theme"),
                        allScheduleActions
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createPostPane(Form postForm) {
        return new FormRenderer(postForm);
    }

    @Override
    protected Form createEditablePutForm(IModel<ScheduleActionDate> entityModel) {
        ScheduleActionDateModel model = (ScheduleActionDateModel) entityModel;
        SingleSelectionField<ScheduleAction> allScheduleActions =
                Field.ofSingleSelectionType(model.allScheduleActionsProperty(),
                                model.allScheduleActionsProperty().indexOf(model.selectedScheduleActionProperty().getValue()))
                        .label("form.label.scheduleaction-all")
                        .required("form.required")
                        .editable(false);
        model.selectedScheduleActionProperty().bind(allScheduleActions.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofIntegerType(model.scheduleActionDateIDProperty())
                                .label("form.label.id")
                                .editable(false),
                        Field.ofDate(model.dateProperty())
                                .label("form.label.date")
                                .required("form.required")
                                .placeholder("form.placeholder.new-date")
                                .editable(false),
                        Field.ofStringType(model.themeProperty())
                                .label("form.label.theme")
                                .required("form.required")
                                .placeholder("form.placeholder.theme")
                                .editable(false),
                        allScheduleActions
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createEditablePutPane(Form editablePutForm) {
        return new FormRenderer(editablePutForm);
    }

    @Override
    protected Form createAdditionalPutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditablePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected Form createAdditionalDeletePutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditableDeletePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected IModel<ScheduleActionDate> createEmptyModel() {
        return new ScheduleActionDateModel();
    }

    @Override
    protected IModel<ScheduleActionDate> createEntityModel(@NotNull ScheduleActionDate entity) {
        IModel<ScheduleActionDate> scheduleActionDateModel = new ScheduleActionDateModel();
        scheduleActionDateModel.importEntity(entity);
        return scheduleActionDateModel;
    }

    @Override
    protected IModel<?> createAdditionalModel() {
        return null;
    }

    @Override
    protected Object getAdditionalEntity(IModel<ScheduleActionDate> additionalModel) {
        return null;
    }
}
