package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform;

import com.dlsc.formsfx.model.structure.Field;
import com.dlsc.formsfx.model.structure.Form;
import com.dlsc.formsfx.model.structure.Group;
import com.dlsc.formsfx.model.structure.SingleSelectionField;
import com.dlsc.formsfx.view.renderer.FormRenderer;
import cz.zcu.fav.kiv.mjakubas.saf.entity.*;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.AEntityController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.IModel;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.ScheduleActionModel;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Represent all possible forms of {@link ScheduleAction} entity.
 */
public class ScheduleActionForm extends AForm<ScheduleAction> {
    @Override
    protected AEntityController<?> getEntityController() {
        return EntitiesController.getEntitiesController().getScheduleActionEntityController();
    }

    @Override
    protected Form createPostForm(IModel<ScheduleAction> entityModel) {
        ScheduleActionModel model = (ScheduleActionModel) entityModel;
        SingleSelectionField<Room> allRoomsField =
                Field.ofSingleSelectionType(model.allRoomsProperty())
                        .label("form.label.rooms-all")
                        .required("form.required");
        model.selectedRoomProperty().bind(allRoomsField.selectionProperty());
        SingleSelectionField<Subject> allSubjectsField =
                Field.ofSingleSelectionType(model.allSubjectsProperty())
                        .label("form.label.subjects-all")
                        .required("form.required");
        model.selectedSubjectProperty().bind(allSubjectsField.selectionProperty());
        SingleSelectionField<SchoolDay> allSchoolDaysField =
                Field.ofSingleSelectionType(model.allSchoolDaysProperty())
                        .label("form.label.schooldays-all")
                        .required("form.required");
        model.selectedSchoolDayProperty().bind(allSchoolDaysField.selectionProperty());
        SingleSelectionField<SchoolHour> allSchoolHoursField =
                Field.ofSingleSelectionType(model.allSchoolHoursProperty())
                        .label("form.label.schoolhours-all")
                        .required("form.required");
        model.selectedSchoolHourProperty().bind(allSchoolHoursField.selectionProperty());

        return Form.of(
                Group.of(
                        allRoomsField,
                        allSubjectsField,
                        allSchoolDaysField,
                        allSchoolHoursField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createPostPane(Form postForm) {
        return new FormRenderer(postForm);
    }

    @Override
    protected Form createEditablePutForm(IModel<ScheduleAction> entityModel) {
        ScheduleActionModel model = (ScheduleActionModel) entityModel;
        SingleSelectionField<Room> allRoomsField =
                Field.ofSingleSelectionType(model.allRoomsProperty(),
                                model.allRoomsProperty().indexOf(model.selectedRoomProperty().getValue()))
                        .label("form.label.rooms-all")
                        .required("form.required");
        model.selectedRoomProperty().bind(allRoomsField.selectionProperty());
        SingleSelectionField<Subject> allSubjectsField =
                Field.ofSingleSelectionType(model.allSubjectsProperty(),
                                model.allSubjectsProperty().indexOf(model.selectedSubjectProperty().getValue()))
                        .label("form.label.subjects-all")
                        .required("form.required");
        model.selectedSubjectProperty().bind(allSubjectsField.selectionProperty());
        SingleSelectionField<SchoolDay> allSchoolDaysField =
                Field.ofSingleSelectionType(model.allSchoolDaysProperty(),
                                model.allSchoolDaysProperty().indexOf(model.selectedSchoolDayProperty().getValue()))
                        .label("form.label.schooldays-all")
                        .required("form.required");
        model.selectedSchoolDayProperty().bind(allSchoolDaysField.selectionProperty());
        SingleSelectionField<SchoolHour> allSchoolHoursField =
                Field.ofSingleSelectionType(model.allSchoolHoursProperty(),
                                model.allSchoolHoursProperty().indexOf(model.selectedSchoolHourProperty().getValue()))
                        .label("form.label.schoolhours-all")
                        .required("form.required");
        model.selectedSchoolHourProperty().bind(allSchoolHoursField.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofIntegerType(model.scheduleActionIDProperty())
                                .label("form.label.id")
                                .editable(false),
                        allRoomsField,
                        allSubjectsField,
                        allSchoolDaysField,
                        allSchoolHoursField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createEditablePutPane(Form editablePutForm) {
        return new FormRenderer(editablePutForm);
    }

    @Override
    protected Form createAdditionalPutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditablePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected Form createAdditionalDeletePutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditableDeletePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected IModel<ScheduleAction> createEmptyModel() {
        return new ScheduleActionModel();
    }

    @Override
    protected IModel<ScheduleAction> createEntityModel(@NotNull ScheduleAction entity) {
        IModel<ScheduleAction> scheduleActionModel = new ScheduleActionModel();
        scheduleActionModel.importEntity(entity);
        return scheduleActionModel;
    }

    @Override
    protected IModel<?> createAdditionalModel() {
        return null;
    }

    @Override
    protected Object getAdditionalEntity(IModel<ScheduleAction> additionalModel) {
        return null;
    }
}
