package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform;

import com.dlsc.formsfx.model.structure.Field;
import com.dlsc.formsfx.model.structure.Form;
import com.dlsc.formsfx.model.structure.Group;
import com.dlsc.formsfx.model.validators.RegexValidator;
import com.dlsc.formsfx.view.renderer.FormRenderer;
import cz.zcu.fav.kiv.mjakubas.saf.entity.SchoolHour;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.AEntityController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.IModel;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.SchoolHourModel;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Represent all possible forms of {@link SchoolHour} entity.
 */
public class SchoolHourForm extends AForm<SchoolHour> {
    @Override
    protected AEntityController<?> getEntityController() {
        return EntitiesController.getEntitiesController().getSchoolHourEntityController();
    }

    @Override
    protected Form createPostForm(IModel<SchoolHour> entityModel) {
        SchoolHourModel model = (SchoolHourModel) entityModel;
        return Form.of(
                Group.of(
                        Field.ofStringType(model.schoolHourNameProperty())
                                .label("form.label.school-hour-name")
                                .required("form.required")
                                .placeholder("form.placeholder.school-hour-name"),
                        Field.ofStringType(model.schoolHourBeginProperty())
                                .label("form.label.school-hour-start")
                                .required("form.required")
                                .placeholder("form.placeholder.school-hour-start")
                                .validate(RegexValidator.forPattern("^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$",
                                        "form.validator.time")),
                        Field.ofStringType(model.schoolHourEndProperty())
                                .label("form.label.school-hour-end")
                                .required("form.required")
                                .placeholder("form.placeholder.school-hour-end")
                                .validate(RegexValidator.forPattern("^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$",
                                        "form.validator.time"))
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createPostPane(Form postForm) {
        return new FormRenderer(postForm);
    }

    @Override
    protected Form createEditablePutForm(IModel<SchoolHour> entityModel) {
        SchoolHourModel model = (SchoolHourModel) entityModel;
        return Form.of(
                Group.of(
                        Field.ofIntegerType(model.schoolHourIDProperty())
                                .label("form.label.id")
                                .editable(false),
                        Field.ofStringType(model.schoolHourNameProperty())
                                .label("form.label.school-hour-name")
                                .required("form.required")
                                .placeholder("form.placeholder.school-hour-name"),
                        Field.ofStringType(model.schoolHourBeginProperty())
                                .label("form.label.school-hour-start")
                                .required("form.required")
                                .placeholder("form.placeholder.school-hour-start")
                                .validate(RegexValidator.forPattern("^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$",
                                        "form.validator.time")),
                        Field.ofStringType(model.schoolHourEndProperty())
                                .label("form.label.school-hour-end")
                                .required("form.required")
                                .placeholder("form.placeholder.school-hour-end")
                                .validate(RegexValidator.forPattern("^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$",
                                        "form.validator.time"))
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createEditablePutPane(Form editablePutForm) {
        return new FormRenderer(editablePutForm);
    }

    @Override
    protected Form createAdditionalPutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditablePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected Form createAdditionalDeletePutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditableDeletePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected IModel<SchoolHour> createEmptyModel() {
        return new SchoolHourModel();
    }

    @Override
    protected IModel<SchoolHour> createEntityModel(@NotNull SchoolHour entity) {
        IModel<SchoolHour> schoolHourModel = new SchoolHourModel();
        schoolHourModel.importEntity(entity);
        return schoolHourModel;
    }

    @Override
    protected IModel<?> createAdditionalModel() {
        return null;
    }

    @Override
    protected Object getAdditionalEntity(IModel<SchoolHour> additionalModel) {
        return null;
    }
}
