package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform;

import com.dlsc.formsfx.model.structure.Field;
import com.dlsc.formsfx.model.structure.Form;
import com.dlsc.formsfx.model.structure.Group;
import com.dlsc.formsfx.model.structure.SingleSelectionField;
import com.dlsc.formsfx.view.renderer.FormRenderer;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Subject;
import cz.zcu.fav.kiv.mjakubas.saf.entity.SubjectList;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Teacher;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.AEntityController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.IModel;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.SubjectModel;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Represent all possible forms of {@link Subject} entity.
 */
public class SubjectForm extends AForm<Subject> {
    @Override
    protected AEntityController<?> getEntityController() {
        return EntitiesController.getEntitiesController().getSubjectEntityController();
    }

    @Override
    protected Form createPostForm(IModel<Subject> entityModel) {
        SubjectModel model = (SubjectModel) entityModel;
        SingleSelectionField<SubjectList> allSubjectListsField =
                Field.ofSingleSelectionType(model.allSubjectListsProperty())
                        .label("form.label.subjects-all")
                        .required("form.required");
        model.selectedSubjectListProperty().bind(allSubjectListsField.selectionProperty());

        SingleSelectionField<Teacher> allTeachersField =
                Field.ofSingleSelectionType(model.allTeachersProperty())
                        .label("form.label.teachers-all")
                        .required("form.required");
        model.selectedTeacherProperty().bind(allTeachersField.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofStringType(model.subjectNameProperty())
                                .label("form.label.subject-name")
                                .required("form.required")
                                .placeholder("form.placeholder.subject-name"),

                        allSubjectListsField,
                        allTeachersField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createPostPane(Form postForm) {
        return new FormRenderer(postForm);
    }

    @Override
    protected Form createEditablePutForm(IModel<Subject> entityModel) {
        SubjectModel model = (SubjectModel) entityModel;
        SingleSelectionField<SubjectList> allSubjectListsField =
                Field.ofSingleSelectionType(model.allSubjectListsProperty(),
                                model.allSubjectListsProperty().indexOf(model.selectedSubjectListProperty().getValue()))
                        .label("form.label.subjects-all")
                        .required("form.required");
        model.selectedSubjectListProperty().bind(allSubjectListsField.selectionProperty());

        SingleSelectionField<Teacher> allTeachersField =
                Field.ofSingleSelectionType(model.allTeachersProperty(),
                                model.allTeachersProperty().indexOf(model.selectedTeacherProperty().getValue()))
                        .label("form.label.teachers-all")
                        .required("form.required");
        model.selectedTeacherProperty().bind(allTeachersField.selectionProperty());

        return Form.of(
                Group.of(
                        Field.ofIntegerType(model.subjectIDProperty())
                                .label("form.label.id")
                                .editable(false),
                        Field.ofStringType(model.subjectNameProperty())
                                .label("form.label.subject-name")
                                .required("form.required")
                                .placeholder("form.placeholder.subject-name"),

                        allSubjectListsField,
                        allTeachersField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createEditablePutPane(Form editablePutForm) {
        return new FormRenderer(editablePutForm);
    }

    @Override
    protected Form createAdditionalPutForm(IModel<?> entityModel) {
        return null;
    }


    @Override
    protected Pane createNonEditablePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected Form createAdditionalDeletePutForm(IModel<?> entityModel) {
        return null;
    }

    @Override
    protected Pane createNonEditableDeletePutPane(Form nonEditablePutForm) {
        return null;
    }

    @Override
    protected IModel<Subject> createEmptyModel() {
        return new SubjectModel();
    }

    @Override
    protected IModel<Subject> createEntityModel(@NotNull Subject entity) {
        IModel<Subject> subjectModel = new SubjectModel();
        subjectModel.importEntity(entity);
        return subjectModel;
    }

    @Override
    protected IModel<?> createAdditionalModel() {
        return null;
    }

    @Override
    protected Object getAdditionalEntity(IModel<Subject> additionalModel) {
        return null;
    }
}
