package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform;

import com.dlsc.formsfx.model.structure.Field;
import com.dlsc.formsfx.model.structure.Form;
import com.dlsc.formsfx.model.structure.Group;
import com.dlsc.formsfx.model.structure.SingleSelectionField;
import com.dlsc.formsfx.model.validators.RegexValidator;
import com.dlsc.formsfx.model.validators.StringLengthValidator;
import com.dlsc.formsfx.view.renderer.FormRenderer;
import cz.zcu.fav.kiv.mjakubas.saf.entity.SubjectList;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Teacher;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.AEntityController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.IModel;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel.TeacherModel;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Represent all possible forms of {@link Teacher} entity.
 */
public class TeacherForm extends AForm<Teacher> {

    @Override
    protected AEntityController<?> getEntityController() {
        return EntitiesController.getEntitiesController().getTeacherEntityController();
    }

    @Override
    protected Form createPostForm(IModel<Teacher> entityModel) {
        TeacherModel model = (TeacherModel) entityModel;
        return Form.of(
                Group.of(
                        Field.ofStringType(model.firstNameProperty())
                                .label("form.label.first-name")
                                .required("form.required")
                                .placeholder("form.placeholder.first-name"),
                        Field.ofStringType(model.lastNameProperty())
                                .label("form.label.last-name")
                                .required("form.required")
                                .placeholder("form.placeholder.last-name"),
                        Field.ofDate(model.birthDateProperty())
                                .label("form.label.birthdate")
                                .required("form.required")
                                .placeholder("form.placeholder.old-date"),
                        Field.ofStringType(model.residenceProperty())
                                .label("form.label.residence")
                                .required("form.required")
                                .placeholder("form.placeholder.residence"),
                        Field.ofStringType(model.phoneProperty())
                                .label("form.label.phone")
                                .required("form.required")
                                .placeholder("form.placeholder.phone")
                                .validate(
                                        StringLengthValidator.exactly(9, "form.validator.phone"),
                                        RegexValidator.forPattern("\\d*", "form.validator.digits")
                                ),
                        Field.ofStringType(model.emailProperty())
                                .label("form.label.email")
                                .required("form.required")
                                .placeholder("form.placeholder.email")
                                .validate(
                                        RegexValidator.forEmail("form.validator.email")
                                ),
                        Field.ofDate(model.commencementDateProperty())
                                .label("form.label.commencement-date")
                                .required("form.required")
                                .placeholder("form.placeholder.new-date")
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createPostPane(Form postForm) {
        return new FormRenderer(postForm);
    }

    @Override
    protected Form createEditablePutForm(IModel<Teacher> entityModel) {
        TeacherModel model = (TeacherModel) entityModel;
        return Form.of(
                Group.of(
                        Field.ofIntegerType(model.idTeachersProperty())
                                .label("form.label.id")
                                .editable(false),
                        Field.ofStringType(model.firstNameProperty())
                                .label("form.label.first-name")
                                .required("form.required")
                                .placeholder("form.placeholder.first-name"),
                        Field.ofStringType(model.lastNameProperty())
                                .label("form.label.last-name")
                                .required("form.required")
                                .placeholder("form.placeholder.last-name"),
                        Field.ofDate(model.birthDateProperty())
                                .label("form.label.birthdate")
                                .required("form.required")
                                .placeholder("form.placeholder.old-date"),
                        Field.ofStringType(model.residenceProperty())
                                .label("form.label.residence")
                                .required("form.required")
                                .placeholder("form.placeholder.residence"),
                        Field.ofStringType(model.phoneProperty())
                                .label("form.label.phone")
                                .required("form.required")
                                .placeholder("form.placeholder.phone")
                                .validate(
                                        StringLengthValidator.exactly(9, "form.validator.phone"),
                                        RegexValidator.forPattern("\\d*", "form.validator.digits")
                                ),
                        Field.ofStringType(model.emailProperty())
                                .label("form.label.email")
                                .required("form.required")
                                .placeholder("form.placeholder.email")
                                .validate(
                                        RegexValidator.forEmail("form.validator.email")
                                ),
                        Field.ofDate(model.commencementDateProperty())
                                .label("form.label.commencement-date")
                                .required("form.required")
                                .placeholder("form.placeholder.new-date")
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createEditablePutPane(Form editablePutForm) {
        return new FormRenderer(editablePutForm);
    }

    @Override
    protected Form createAdditionalPutForm(IModel<?> entityModel) {
        TeacherModel model = (TeacherModel) entityModel;
        SingleSelectionField<SubjectList> allSubjectListsField =
                Field.ofSingleSelectionType(model.allSubjectListsProperty())
                        .label("form.label.subjects-all")
                        .required("form.required");
        model.selectedSubjectListProperty().bind(allSubjectListsField.selectionProperty());

        return Form.of(
                Group.of(
                        allSubjectListsField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createNonEditablePutPane(Form nonEditablePutForm) {
        return new FormRenderer(nonEditablePutForm);
    }

    @Override
    protected Form createAdditionalDeletePutForm(IModel<?> entityModel) {
        TeacherModel model = (TeacherModel) entityModel;
        SingleSelectionField<SubjectList> allSubjectListsField =
                Field.ofSingleSelectionType(model.allTeacherSubjectListsProperty())
                        .label("form.label.subjects-all")
                        .required("form.required");
        model.selectedSubjectListProperty().bind(allSubjectListsField.selectionProperty());

        return Form.of(
                Group.of(
                        allSubjectListsField
                )
        ).i18n(I18N.getLocalizationTool().getResourceBundleService());
    }

    @Override
    protected Pane createNonEditableDeletePutPane(Form nonEditablePutForm) {
        return new FormRenderer(nonEditablePutForm);
    }

    @Override
    protected IModel<Teacher> createEmptyModel() {
        return new TeacherModel();
    }

    @Override
    protected IModel<Teacher> createEntityModel(@NotNull Teacher entity) {
        IModel<Teacher> teacherModel = new TeacherModel();
        teacherModel.importEntity(entity);
        return teacherModel;
    }

    @Override
    protected IModel<?> createAdditionalModel() {
        return null;
    }

    @Override
    protected Object getAdditionalEntity(IModel<Teacher> additionalModel) {
        TeacherModel model = (TeacherModel) additionalModel;
        return model.selectedSubjectListProperty().getValue();
    }
}
