package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel;

import cz.zcu.fav.kiv.mjakubas.saf.entity.*;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.FetchException;
import javafx.beans.property.*;
import javafx.collections.FXCollections;

import java.util.List;


public class ClassModel implements IModel<SchoolClass> {

    private final IntegerProperty classID = new SimpleIntegerProperty(0);
    private final StringProperty className = new SimpleStringProperty("");
    private final ListProperty<Room> allRooms = new SimpleListProperty<>(FXCollections.observableArrayList());
    private final ListProperty<Teacher> allTeacher = new SimpleListProperty<>(FXCollections.observableArrayList());

    private final ObjectProperty<Room> selectedRoom = new SimpleObjectProperty<>();
    private final ObjectProperty<Teacher> selectedTeacher = new SimpleObjectProperty<>();

    private final ListProperty<Subject> allSubjects = new SimpleListProperty<>(FXCollections.observableArrayList());
    private final ObjectProperty<Subject> selectedSubjects = new SimpleObjectProperty<>();
    private final ListProperty<Subject> allTeacherSubjects = new SimpleListProperty<>(FXCollections.observableArrayList());

    public ClassModel() {
        makeAdditionalRequests();
    }

    // override

    @Override
    public void importEntity(SchoolClass entity) {
        classID.set(entity.getIdClasses());
        className.set(entity.getClassName());
        selectedRoom.set(entity.getRoom());
        selectedTeacher.set(entity.getTeacher());
        this.allTeacherSubjects.addAll(entity.getSubjects());
    }

    @Override
    public SchoolClass exportEntity() {
        SchoolClass schoolClass = new SchoolClass();
        schoolClass.setIdClasses(classID.getValue());
        schoolClass.setClassName(className.getValue());
        schoolClass.setRoom(selectedRoom.getValue());
        schoolClass.setTeacher(selectedTeacher.getValue());
        return schoolClass;
    }

    @Override
    public void fillAdditionalEntities() throws FetchException {
        EntitiesController entitiesController = EntitiesController.getEntitiesController();

        List<Room> roomList =
                entitiesController.getRoomEntityController().getAllFetchedEntities();
        List<Teacher> teacherList =
                entitiesController.getTeacherEntityController().getAllFetchedEntities();

        allRooms.addAll(roomList);
        allTeacher.addAll(teacherList);

        List<Subject> subjectList =
                entitiesController.getSubjectEntityController().getAllFetchedEntities();

        this.allSubjects.addAll(subjectList);
    }

    // get

    /**
     * Gets class entity id.
     *
     * @return class entity id
     */
    public IntegerProperty classIDProperty() {
        return classID;
    }

    /**
     * Gets class entity name.
     *
     * @return class entity name
     */
    public StringProperty classNameProperty() {
        return className;
    }

    /**
     * Gets all room entities.
     *
     * @return all room entities.
     */
    public ListProperty<Room> allRoomsProperty() {
        return allRooms;
    }

    /**
     * Gets all teacher entities.
     *
     * @return all teacher entities
     */
    public ListProperty<Teacher> allTeacherProperty() {
        return allTeacher;
    }

    /**
     * Gets selected room entity.
     *
     * @return selected room entity
     */
    public ObjectProperty<Room> selectedRoomProperty() {
        return selectedRoom;
    }

    /**
     * Gets selected teacher entity.
     *
     * @return selected teacher entity
     */
    public ObjectProperty<Teacher> selectedTeacherProperty() {
        return selectedTeacher;
    }

    public ListProperty<Subject> allSubjectsProperty() {
        return allSubjects;
    }

    public ObjectProperty<Subject> selectedSubjectsProperty() {
        return selectedSubjects;
    }

    public ListProperty<Subject> allTeacherSubjectsProperty() {
        return allTeacherSubjects;
    }
}
