package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel;

import cz.zcu.fav.kiv.mjakubas.saf.entity.Evaluation;
import cz.zcu.fav.kiv.mjakubas.saf.entity.StudentsSubject;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.FetchException;
import javafx.beans.property.*;
import javafx.collections.FXCollections;

import java.util.List;

/**
 * Represents a model object of an evaluation entity.
 */
public class EvaluationModel implements IModel<Evaluation> {

    private final IntegerProperty evaluationID = new SimpleIntegerProperty(0);
    private final StringProperty testName = new SimpleStringProperty("");
    private final DoubleProperty grade = new SimpleDoubleProperty(0);
    private final IntegerProperty weight = new SimpleIntegerProperty(0);
    private final ListProperty<StudentsSubject> allStudentsSubjects = new SimpleListProperty<>(FXCollections.observableArrayList());
    private final ObjectProperty<StudentsSubject> selectedStudentsSubject = new SimpleObjectProperty<>();

    public EvaluationModel() {
        makeAdditionalRequests();
    }

    // override

    @Override
    public void importEntity(Evaluation entity) {
        evaluationID.set(entity.getID());
        testName.set(entity.getTestName());
        grade.set(entity.getGrade());
        weight.set(entity.getWeight());
        selectedStudentsSubject.set(entity.getStudentssubject());
    }

    @Override
    public Evaluation exportEntity() {
        Evaluation evaluation = new Evaluation();
        evaluation.setIdEvaluations(evaluationID.getValue());
        evaluation.setTestName(testName.getValue());
        evaluation.setGrade((float) grade.get());
        evaluation.setWeight(weight.get());
        evaluation.setStudentssubject(selectedStudentsSubject.getValue());
        return evaluation;
    }

    @Override
    public void fillAdditionalEntities() throws FetchException {
        EntitiesController entitiesController = EntitiesController.getEntitiesController();

        List<StudentsSubject> studentsSubjectList =
                entitiesController.getStudentsSubjectEntityController().getAllFetchedEntities();

        allStudentsSubjects.addAll(studentsSubjectList);
    }

    // get

    /**
     * Gets evaluation entity id.
     *
     * @return evaluation entity id
     */
    public IntegerProperty evaluationIDProperty() {
        return evaluationID;
    }

    /**
     * Gets evaluation entity name.
     *
     * @return evaluation entity name
     */
    public StringProperty testNameProperty() {
        return testName;
    }

    /**
     * Gets evaluation entity grade.
     *
     * @return evaluation entity grade
     */
    public DoubleProperty gradeProperty() {
        return grade;
    }

    /**
     * Gets evaluation entity weight.
     *
     * @return evaluation entity weight
     */
    public IntegerProperty weightProperty() {
        return weight;
    }

    /**
     * Gets all students subjects entities.
     *
     * @return all students subjects entities
     */
    public ListProperty<StudentsSubject> allStudentsSubjectsProperty() {
        return allStudentsSubjects;
    }

    /**
     * Gets selected student subject entity.
     *
     * @return selected student subject entity
     */
    public ObjectProperty<StudentsSubject> selectedStudentsSubjectProperty() {
        return selectedStudentsSubject;
    }
}
