package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel;

import cz.zcu.fav.kiv.mjakubas.saf.entity.Building;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Room;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.FetchException;
import javafx.beans.property.*;
import javafx.collections.FXCollections;

import java.util.List;

/**
 * Represents a model object of a room entity.
 */
public class RoomModel implements IModel<Room> {

    private final IntegerProperty roomID = new SimpleIntegerProperty(0);
    private final StringProperty roomName = new SimpleStringProperty("");
    private final IntegerProperty capacity = new SimpleIntegerProperty(0);
    private final ListProperty<Building> allBuildings = new SimpleListProperty<>(FXCollections.observableArrayList());

    private final ObjectProperty<Building> selectedBuilding = new SimpleObjectProperty<>();

    // constructor

    public RoomModel() {
        makeAdditionalRequests();
    }

    // override

    @Override
    public void importEntity(Room entity) {
        roomID.set(entity.getID());
        roomName.set(entity.getRoomName());
        capacity.set(entity.getCapacity());
        selectedBuilding.set(entity.getBuilding());
    }

    @Override
    public Room exportEntity() {
        Room room = new Room();
        room.setIdRooms(roomID.getValue());
        room.setRoomName(roomName.getValue());
        room.setCapacity(capacity.getValue());
        room.setBuilding(selectedBuilding.getValue());
        return room;
    }

    @Override
    public void fillAdditionalEntities() throws FetchException {
        EntitiesController entitiesController = EntitiesController.getEntitiesController();

        List<Building> buildingList =
                entitiesController.getBuildingEntityController().getAllFetchedEntities();
        allBuildings.addAll(buildingList);
    }

    //get

    /**
     * Gets room entity id.
     *
     * @return room entity id
     */
    public IntegerProperty roomIDProperty() {
        return roomID;
    }

    /**
     * Gets room entity name.
     *
     * @return room entity name
     */
    public StringProperty roomNameProperty() {
        return roomName;
    }

    /**
     * Gets room entity capacity.
     *
     * @return room entity capacity
     */
    public IntegerProperty capacityProperty() {
        return capacity;
    }

    /**
     * Gets all building entities.
     *
     * @return all building entities
     */
    public ListProperty<Building> allBuildingsProperty() {
        return allBuildings;
    }

    /**
     * Gets selected building entity.
     *
     * @return selected building entity
     */
    public ObjectProperty<Building> selectedBuildingProperty() {
        return selectedBuilding;
    }
}
