package cz.zcu.fav.kiv.mjakubas.saf.gui.form.nmodel;

import cz.zcu.fav.kiv.mjakubas.saf.entity.SubjectList;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Teacher;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.FetchException;
import javafx.beans.property.*;
import javafx.collections.FXCollections;

import java.time.LocalDate;
import java.util.List;

/**
 * Represents a model object of a teacher entity.
 */
public class TeacherModel implements IModel<Teacher> {

    private final IntegerProperty idTeachers = new SimpleIntegerProperty(0);
    private final StringProperty firstName = new SimpleStringProperty("");
    private final StringProperty lastName = new SimpleStringProperty("");
    private final ObjectProperty<LocalDate> birthDate = new SimpleObjectProperty<>();
    private final ObjectProperty<LocalDate> commencementDate = new SimpleObjectProperty<>();
    private final StringProperty residence = new SimpleStringProperty("");
    private final StringProperty phone = new SimpleStringProperty("");
    private final StringProperty email = new SimpleStringProperty("");
    private final ListProperty<SubjectList> allSubjectLists = new SimpleListProperty<>(FXCollections.observableArrayList());
    private final ObjectProperty<SubjectList> selectedSubjectList = new SimpleObjectProperty<>();
    private final ListProperty<SubjectList> allTeacherSubjectLists = new SimpleListProperty<>(FXCollections.observableArrayList());

    public TeacherModel() {
        makeAdditionalRequests();
    }

    // override

    @Override
    public void importEntity(Teacher entity) {
        this.idTeachersProperty().set(entity.getIdTeachers());
        this.firstNameProperty().set(entity.getFirstName());
        this.lastNameProperty().set(entity.getLastName());
        this.birthDateProperty().set(LocalDate.parse(entity.getBirthDate()));
        this.commencementDateProperty().set(LocalDate.parse(entity.getCommencementDate()));
        this.residenceProperty().set(entity.getResidence());
        this.phoneProperty().set("" + entity.getPhone());
        this.emailProperty().set(entity.getEmail());
        this.allTeacherSubjectLists.addAll(entity.getSubjectlists());
    }

    @Override
    public Teacher exportEntity() {
        return new Teacher(idTeachers.getValue(), firstName.getValue(), lastName.getValue(),
                birthDate.getValue().toString(), commencementDate.getValue().toString(), residence.getValue(),
                Integer.parseInt(phone.getValue()), email.getValue(), null, null, null);
    }

    @Override
    public void fillAdditionalEntities() throws FetchException {
        EntitiesController entitiesController = EntitiesController.getEntitiesController();

        List<SubjectList> subjectList =
                entitiesController.getSubjectListEntityController().getAllFetchedEntities();

        this.allSubjectLists.addAll(subjectList);
    }

    // get

    /**
     * Gets teacher entity id.
     *
     * @return teacher entity id
     */
    public IntegerProperty idTeachersProperty() {
        return idTeachers;
    }

    /**
     * Gets teacher entity first name.
     *
     * @return teacher entity first name
     */
    public StringProperty firstNameProperty() {
        return firstName;
    }

    /**
     * Gets teacher entity last name.
     *
     * @return teacher entity last name
     */
    public StringProperty lastNameProperty() {
        return lastName;
    }

    /**
     * Gets teacher entity birthdate.
     *
     * @return teacher entity birthdate
     */
    public ObjectProperty<LocalDate> birthDateProperty() {
        return birthDate;
    }

    /**
     * Gets teacher entity commencement date.
     *
     * @return teacher entity commencement date
     */
    public ObjectProperty<LocalDate> commencementDateProperty() {
        return commencementDate;
    }

    /**
     * Gets teacher entity residence.
     *
     * @return teacher entity residence
     */
    public StringProperty residenceProperty() {
        return residence;
    }

    /**
     * Gets teacher entity phone.
     *
     * @return teacher entity phone
     */
    public StringProperty phoneProperty() {
        return phone;
    }

    /**
     * Gets teacher entity email.
     *
     * @return teacher entity email
     */
    public StringProperty emailProperty() {
        return email;
    }

    public ListProperty<SubjectList> allSubjectListsProperty() {
        return allSubjectLists;
    }

    public ObjectProperty<SubjectList> selectedSubjectListProperty() {
        return selectedSubjectList;
    }

    public ListProperty<SubjectList> allTeacherSubjectListsProperty() {
        return allTeacherSubjectLists;
    }
}
