package cz.zcu.fav.kiv.mjakubas.saf.gui.menu;

import javafx.beans.binding.StringBinding;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuItem;

import java.util.List;

/**
 * Represents a {@link Menu} GUI element implementation of certain content.
 */
public abstract class AMenu {

    private Menu menu;

    private final StringBinding menuName;

    /**
     * Instantiates new class representing a {@link Menu} of certain content.
     *
     * @param menuName menu name
     */
    public AMenu(StringBinding menuName) {
        this.menuName = menuName;
    }

    private void createMenu() {
        menu = new Menu();
        menu.textProperty().bind(menuName);

        menu.getItems().addAll(defineMenuItems());
    }

    // abstract

    /**
     * Defines all {@link Menu} {@link MenuItem} elements.
     *
     * @return {@link Menu} {@link MenuItem} elements
     */
    protected abstract List<MenuItem> defineMenuItems();

    // get

    /**
     * Gets class implementation of {@link Menu}.
     *
     * @return class implementation of {@link Menu}
     */
    public Menu getMenu() {
        if (menu == null) {
            createMenu();
        }

        return menu;
    }
}
