package cz.zcu.fav.kiv.mjakubas.saf.gui.view;

import cz.zcu.fav.kiv.mjakubas.saf.gui.i18n.I18N;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.category.ACategoryHandler;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.category.categories.*;
import cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity.EntitiesController;
import javafx.beans.binding.StringBinding;
import javafx.geometry.Pos;
import javafx.scene.control.ToggleButton;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

/**
 * Class provides a {@link Pane} view of a searchable list of all categories as a list of {@link ToggleButton}.
 */
public class CategorySearchableButtonPane extends ASearchableButtonPane<ToggleButton> {

    // override
    @Override
    protected StringBinding getSearchFieldPrompt() {
        return I18N.getLocalizationTool().createStringBinding("category.prompt.search");
    }

    @Override
    protected StringBinding getEmptyButtonLabel() {
        return I18N.getLocalizationTool().createStringBinding("category.label.empty");
    }

    @Override
    protected Pane fillOptionalPane() {
        return null;
    }

    // category

    /**
     * Adds new category to this {@link CategorySearchableButtonPane}.
     *
     * @param categoryName    category name binding
     * @param categoryHandler specific category handler
     */
    public void addCategory(@NotNull StringBinding categoryName, @NotNull ACategoryHandler<?, ?, ?> categoryHandler) {
        ToggleButton categoryButton = new ToggleButton();
        categoryButton.textProperty().bind(categoryName);

        categoryButton.setAlignment(Pos.BASELINE_LEFT);
        //categoryButton.getStyleClass().addAll("btn", "btn-lg", "btn-custom");
        categoryButton.getStyleClass().addAll("btn-custom-primary");
        categoryButton.getStyleClass().addAll("middle"); // needs to be separated because otherwise is ignored

        this.addSearchableButton(categoryButton, () -> {
            categoryHandler.getViewPane().setVisible(true);
            categoryHandler.doSelect();
        });
    }

    // constructor

    /**
     * Creates all categories.
     */
    private void buildCategories(EntitiesController entitiesController, EntitiesSearchableButtonPane viewPane, EntityFormsPane formsPane) {
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.absences"),
                new AbsenceCategoryHandler(entitiesController.getAbsenceEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.buildings"),
                new BuildingCategoryHandler(entitiesController.getBuildingEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.classes"),
                new ClassCategoryHandler(entitiesController.getClassEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.evaluations"),
                new EvaluationCategoryHandler(entitiesController.getEvaluationEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.rooms"),
                new RoomCategoryHandler(entitiesController.getRoomEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.schedule-actions-dates"),
                new ScheduleActionDateCategoryHandler(entitiesController.getScheduleActionDateEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.schedule-actions"),
                new ScheduleActionCategoryHandler(entitiesController.getScheduleActionEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.school-days"),
                new SchoolDayCategoryHandler(entitiesController.getSchoolDayEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.school-hours"),
                new SchoolHourCategoryHandler(entitiesController.getSchoolHourEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.students"),
                new StudentCategoryHandler(entitiesController.getStudentEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.student-subjects"),
                new StudentsSubjectCategoryHandler(entitiesController.getStudentsSubjectEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.subjects"),
                new SubjectCategoryHandler(entitiesController.getSubjectEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.subject-lists"),
                new SubjectListCategoryHandler(entitiesController.getSubjectListEntityController(), viewPane, formsPane)
        );
        addWrappedCategory(
                I18N.getLocalizationTool().createStringBinding("category.label.teachers"),
                new TeacherCategoryHandler(entitiesController.getTeacherEntityController(), viewPane, formsPane)
        );
    }

    // addCategory wrapper

    private void addWrappedCategory(StringBinding categoryName, ACategoryHandler<?, ?, ?> categoryHandler) {
        addCategory(categoryName, categoryHandler);
    }

    /**
     * Instantiates new {@link ASearchableButtonPane} of categories.
     */
    public CategorySearchableButtonPane(@NotNull EntitiesController entitiesController, @NotNull EntitiesSearchableButtonPane entitiesSearchableButtonPane, @NotNull EntityFormsPane formsPane) {
        super();
        buildCategories(entitiesController, entitiesSearchableButtonPane, formsPane);
    }
}
