package cz.zcu.fav.kiv.mjakubas.saf.request;

import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.*;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.IMapper;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.JSONMapper;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.XMLMapper;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.exception.MapperException;
import org.jetbrains.annotations.NotNull;

import java.util.*;

/**
 * Allows posting multiple entities of different type in the same file.
 */
public class MultiPost {

    public static final Map<Class<?>, String> allEntities = new HashMap<>();

    static {
        allEntities.putIfAbsent(Absence.class, RequestConfig.ABSENCE_ENTITIES_QUERY);
        allEntities.putIfAbsent(Building.class, RequestConfig.BUILDING_ENTITIES_QUERY);
        allEntities.putIfAbsent(Evaluation.class, RequestConfig.EVALUATIONS_ENTITIES_QUERY);
        allEntities.putIfAbsent(Room.class, RequestConfig.ROOMS_ENTITIES_QUERY);
        allEntities.putIfAbsent(ScheduleAction.class, RequestConfig.SCHEDULEACTIONS_ENTITIES_QUERY);
        allEntities.putIfAbsent(ScheduleActionDate.class, RequestConfig.SCHEDULEACTIONDATES_ENTITIES_QUERY);
        allEntities.putIfAbsent(SchoolClass.class, RequestConfig.CLASSES_ENTITIES_QUERY);
        allEntities.putIfAbsent(SchoolDay.class, RequestConfig.SCHOOLDAYS_ENTITIES_QUERY);
        allEntities.putIfAbsent(SchoolHour.class, RequestConfig.SCHOOLHOURS_ENTITIES_QUERY);
        allEntities.putIfAbsent(Student.class, RequestConfig.STUDENTS_ENTITIES_QUERY);
        allEntities.putIfAbsent(StudentsSubject.class, RequestConfig.STUDENTSSUBJECTS_ENTITIES_QUERY);
        allEntities.putIfAbsent(Subject.class, RequestConfig.SUBJECTS_ENTITIES_QUERY);
        allEntities.putIfAbsent(SubjectList.class, RequestConfig.SUBJECTLISTS_ENTITIES_QUERY);
        allEntities.putIfAbsent(Teacher.class, RequestConfig.TEACHERS_ENTITIES_QUERY);
    }

    private static boolean identifyXML(@NotNull String entities) {
        return entities.charAt(0) == '<';
    }

    private static boolean identifyJSON(@NotNull String entities) {
        return entities.charAt(0) == '[';
    }

    private static String[] parseXML(@NotNull String entities) {
        String preprocessed = entities.substring(9, entities.length() - 10); // remove objects tag
        String[] processed = preprocessed.split("<Group>");
        processed = Arrays.copyOfRange(processed, 1, processed.length);
        for (int i = 0; i < processed.length; i++) {
            processed[i] = processed[i].replaceAll(" *<", "<");
            processed[i] = processed[i].replaceAll("> *", ">");
            processed[i] = processed[i].replaceAll(">(\r\n)*", ">");
            processed[i] = processed[i].replaceAll("\s\s+", "");
            processed[i] = processed[i].substring(0, processed[i].length() - 8);
        }

        return processed;
    }

    private static String[] parseJSON(@NotNull String entities) {
        String preprocessed = entities.substring(1, entities.length() - 2);
        String[] processed = preprocessed.split("\\[");
        processed = Arrays.copyOfRange(processed, 1, processed.length);
        for (int i = 0; i < processed.length; i++) {
            processed[i] = "[" + processed[i];
        }

        return processed;
    }

    /**
     * Processes a serialized string of files in accepted format to produce
     * a map of entities with their base address for posting entities to back-end.
     * @param entities serialized string of entities
     * @return a map of entities with key as their base post address
     */
    public static HashMap<String, List<Object>> processEntities(@NotNull String entities) {
        IMapper mapper = null;
        String[] parsedEntities = null;
        if (identifyJSON(entities)) {
            mapper = new JSONMapper();
            parsedEntities = parseJSON(entities);
        }
        if (identifyXML(entities)) {
            mapper = new XMLMapper();
            parsedEntities = parseXML(entities);
        }
        if (mapper == null || parsedEntities == null)
            return null;

        HashMap<String, List<Object>> processed = new HashMap<>();
        for (String p : parsedEntities) {
            boolean success = false;
            for (Class<?> key : allEntities.keySet()) {
                try {
                    Object[] os = mapper.deserialize(p, key);
                    success = true;
                    processed.putIfAbsent(allEntities.get(key), new ArrayList<>());
                    for (Object o : os) {
                        processed.get(allEntities.get(key)).add(o);
                    }
                    break;
                } catch (MapperException e) {
                    //ignored
                }
            }
            if (!success)
                return null;
        }

        return processed;
    }
}
