package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.RequestConfigTestConfigTest;
import cz.zcu.fav.kiv.mjakubas.saf.TestEntitiesPosterTest;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Subject;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.Objects;

import static org.junit.jupiter.api.Assertions.*;

class EvaluationsCommandsHandlerTest {

    private static final String EVALUATIONS_TEST_FILE_NAME = "test-evaluations";

    protected EvaluationsCommandsHandler h = new EvaluationsCommandsHandler(RequestConfigTestConfigTest.getTestConfig());
    protected int[][] ids;

    @BeforeEach
    void setUp() throws Exception {
        ids = TestEntitiesPosterTest.postTestEvaluations();
    }

    @AfterEach
    void tearDown() throws Exception {
        TestEntitiesPosterTest.deleteTestEvaluations(ids);
    }

    @Test
    void handleGetAllJSONTest() throws Exception {
        h.handleGetAll(FileType.JSON, EVALUATIONS_TEST_FILE_NAME);
        String expected = EntitiesStringComparatorTest.getExpectedContentString("commands/evaluation/test-evaluations.json");
        expected = expected.replaceAll("-1", "" + ids[0][0]);
        expected = expected.replaceAll("-2", "" + ids[0][1]);

        String actual = Files.readString(Path.of(EVALUATIONS_TEST_FILE_NAME));
        assertEquals(expected, actual);
    }

    @Test
    void handleGetAllXMLTest() throws Exception {
        h.handleGetAll(FileType.XML, EVALUATIONS_TEST_FILE_NAME);
        String expected = EntitiesStringComparatorTest.getExpectedContentString("commands/evaluation/test-evaluations.xml");
        expected = expected.replaceAll("-1", "" + ids[0][0]);
        expected = expected.replaceAll("-2", "" + ids[0][1]);

        String actual = Files.readString(Path.of(EVALUATIONS_TEST_FILE_NAME));
        assertEquals(expected, actual);
    }

    @Test
    void handleGetAllJSONTestEmpty() throws Exception {
        tearDown();
        ids = new int[][]{new int[]{}};

        h.handleGetAll(FileType.JSON, EVALUATIONS_TEST_FILE_NAME);
        String expected = EntitiesStringComparatorTest.getExpectedContentString("commands/test-empty.json");

        String actual = Files.readString(Path.of(EVALUATIONS_TEST_FILE_NAME));
        assertEquals(expected, actual);
    }

    @Test
    void handleGetAllXMLTestEmpty() throws Exception {
        tearDown();
        ids = new int[][]{new int[]{}};

        h.handleGetAll(FileType.XML, EVALUATIONS_TEST_FILE_NAME);
        String expected = EntitiesStringComparatorTest.getExpectedContentString("commands/evaluation/test-empty-evaluations.xml");

        String actual = Files.readString(Path.of(EVALUATIONS_TEST_FILE_NAME));
        assertEquals(expected, actual);
    }

    // --

    @Test
    void handlePostJSONSuccessful() throws Exception {
        Subject sub = (Subject) EntitiesRequests.get(RequestConfigTestConfigTest.getTestConfig(),
                RequestConfig.SUBJECTS_ENTITIES_QUERY,
                Subject.class).getDeserializedEntities()[0];
        int id = sub.getStudentssubjects()[0].getID();

        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/json/test-post-evaluation.json"))
                        .getPath();
        String s = Files.readString(new File(path).toPath()).replaceAll("1a", "" + id);
        File file = File.createTempFile("temp", "test-frontend");
        Files.writeString(file.toPath(), s);

        h.handlePost(FileType.JSON, file.getAbsolutePath());

        int[] ms = Arrays.copyOf(ids[0], ids[0].length + 1);
        ms[ms.length - 1] = ms[ms.length - 2] + 1;
        ids[0] = ms;
    }

    @Test
    void handlePostXMLSuccessful() throws Exception {
        Subject sub = (Subject) EntitiesRequests.get(RequestConfigTestConfigTest.getTestConfig(),
                RequestConfig.SUBJECTS_ENTITIES_QUERY,
                Subject.class).getDeserializedEntities()[0];
        int id = sub.getStudentssubjects()[0].getID();

        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/xml/test-post-evaluation.xml"))
                        .getPath();
        String s = Files.readString(new File(path).toPath()).replaceAll("1a", "" + id);
        File file = File.createTempFile("temp", "test-frontend");
        Files.writeString(file.toPath(), s);

        h.handlePost(FileType.XML, file.getAbsolutePath());

        int[] ms = Arrays.copyOf(ids[0], ids[0].length + 1);
        ms[ms.length - 1] = ms[ms.length - 2] + 1;
        ids[0] = ms;
    }

    @Test
    void handleDelete() throws Exception {
        Subject sub = (Subject) EntitiesRequests.get(RequestConfigTestConfigTest.getTestConfig(),
                RequestConfig.SUBJECTS_ENTITIES_QUERY,
                Subject.class).getDeserializedEntities()[0];
        int id = sub.getStudentssubjects()[0].getID();

        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/json/test-post-evaluation.json"))
                        .getPath();
        String s = Files.readString(new File(path).toPath()).replaceAll("1a", "" + id);
        File file = File.createTempFile("temp", "test-frontend");
        Files.writeString(file.toPath(), s);

        h.handlePost(FileType.JSON, file.getAbsolutePath());
        h.handleDeleteByID(ids[0][ids[0].length - 1] + 1);
    }

    @Test
    void handleDeleteVoid() {
        Assertions.assertThrows(CommandException.class, () -> h.handleDeleteByID(0));
    }

    // --

    @Test
    void handlePostJSONUnsuccessful1() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/json/test-post-evaluation-e1" +
                                        ".json"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.JSON, path));
    }

    @Test
    void handlePostJSONUnsuccessful2() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/json/test-post-evaluation-e2" +
                                        ".json"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.JSON, path));
    }

    @Test
    void handlePostJSONUnsuccessful3() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/json/test-post-evaluation-e3" +
                                        ".json"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.JSON, path));
    }

    @Test
    void handlePostJSONUnsuccessful4() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/json/test-post-evaluation-e4" +
                                        ".json"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.JSON, path));
    }

    @Test
    void handlePostXMLUnsuccessful1() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/xml/test-post-evaluation-e1.xml"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.XML, path));
    }

    @Test
    void handlePostXMLUnsuccessful2() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/xml/test-post-evaluation-e2" +
                                        ".xml"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.XML, path));
    }

    @Test
    void handlePostXMLUnsuccessful3() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/xml/test-post-evaluation-e3" +
                                        ".xml"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.XML, path));
    }

    @Test
    void handlePostXMLUnsuccessful4() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/evaluation/post/xml/test-post-evaluation-e4" +
                                        ".xml"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.XML, path));
    }
}