package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.RequestConfigTestConfigTest;
import cz.zcu.fav.kiv.mjakubas.saf.TestEntitiesPosterTest;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.Objects;

import static org.junit.jupiter.api.Assertions.*;

class SubjectsCommandsHandlerTest {

    private static final String SUBJECTS_TEST_FILE_NAME = "test-subjects";

    protected SubjectsCommandsHandler h = new SubjectsCommandsHandler(RequestConfigTestConfigTest.getTestConfig());
    protected int[][] ids;

    @BeforeEach
    void setUp() throws Exception {
        ids = TestEntitiesPosterTest.postTestSubjects();
    }

    @AfterEach
    void tearDown() throws Exception {
        TestEntitiesPosterTest.deleteTestSubjects(ids);
    }

    @Test
    void handleGetAllJSONTest() throws Exception {
        h.handleGetAll(FileType.JSON, SUBJECTS_TEST_FILE_NAME);
        String expected = EntitiesStringComparatorTest.getExpectedContentString("commands/subject/test-subjects.json");
        expected = expected.replaceAll("-1", "" + ids[0][0]);
        expected = expected.replaceAll("-2", "" + ids[0][1]);

        String actual = Files.readString(Path.of(SUBJECTS_TEST_FILE_NAME));
        assertEquals(expected, actual);
    }

    @Test
    void handleGetAllXMLTest() throws Exception {
        h.handleGetAll(FileType.XML, SUBJECTS_TEST_FILE_NAME);
        String expected = EntitiesStringComparatorTest.getExpectedContentString("commands/subject/test-subjects.xml");
        expected = expected.replaceAll("-1", "" + ids[0][0]);
        expected = expected.replaceAll("-2", "" + ids[0][1]);

        String actual = Files.readString(Path.of(SUBJECTS_TEST_FILE_NAME));
        assertEquals(expected, actual);
    }

    @Test
    void handleGetAllJSONTestEmpty() throws Exception {
        tearDown();
        ids = new int[][]{new int[]{}};

        h.handleGetAll(FileType.JSON, SUBJECTS_TEST_FILE_NAME);
        String expected = EntitiesStringComparatorTest.getExpectedContentString("commands/test-empty.json");

        String actual = Files.readString(Path.of(SUBJECTS_TEST_FILE_NAME));
        assertEquals(expected, actual);
    }

    @Test
    void handleGetAllXMLTestEmpty() throws Exception {
        tearDown();
        ids = new int[][]{new int[]{}};

        h.handleGetAll(FileType.XML, SUBJECTS_TEST_FILE_NAME);
        String expected = EntitiesStringComparatorTest.getExpectedContentString("commands/subject/test-empty-subjects.xml");

        String actual = Files.readString(Path.of(SUBJECTS_TEST_FILE_NAME));
        assertEquals(expected, actual);
    }

    // --

    @Test
    void handlePostJSONSuccessful() throws Exception {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/json/test-post-subject.json"))
                        .getPath();
        h.handlePost(FileType.JSON, path);

        int[] ms = Arrays.copyOf(ids[0], ids[0].length + 1);
        ms[ms.length - 1] = ms[ms.length - 2] + 1;
        ids[0] = ms;
    }

    @Test
    void handlePostXMLSuccessful() throws Exception {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/xml/test-post-subject.xml"))
                        .getPath();

        h.handlePost(FileType.XML, path);

        int[] ms = Arrays.copyOf(ids[0], ids[0].length + 1);
        ms[ms.length - 1] = ms[ms.length - 2] + 1;
        ids[0] = ms;
    }

    @Test
    void handleDelete() throws Exception {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/xml/test-post-subject.xml"))
                        .getPath();
        h.handlePost(FileType.XML, path);
        h.handleDeleteByID(ids[0][ids[0].length - 1] + 1);
    }

    @Test
    void handleDeleteVoid() {
        Assertions.assertThrows(CommandException.class, () -> h.handleDeleteByID(0));
    }

    // --

    @Test
    void handlePostJSONUnsuccessful1() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/json/test-post-subject-e1.json"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.JSON, path));
    }

    @Test
    void handlePostJSONUnsuccessful2() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/json/test-post-subject-e2.json"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.JSON, path));
    }

    @Test
    void handlePostJSONUnsuccessful3() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/json/test-post-subject-e3.json"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.JSON, path));
    }

    @Test
    void handlePostXMLUnsuccessful1() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/xml/test-post-subject-e1.xml"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.XML, path));
    }

    @Test
    void handlePostXMLUnsuccessful2() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/xml/test-post-subject-e2.xml"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.XML, path));
    }

    @Test
    void handlePostXMLUnsuccessful3() {
        String path =
                Objects.requireNonNull(this.getClass()
                                .getClassLoader().getResource("commands/subject/post/xml/test-post-subject-e3.xml"))
                        .getPath();
        Assertions.assertThrows(CommandException.class, () -> h.handlePost(FileType.XML, path));
    }
}