package cz.zcu.fav.kiv.mjakubas.saf.io.export;

import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Objects;

import static org.junit.jupiter.api.Assertions.assertEquals;

class EntityExporterTest {

    A a;
    Path jsonR;
    Path jsonRM;

    Path jsonX;
    Path jsonXM;

    @BeforeEach
    void setUp() throws Exception {
        a = new A(10, "text");
        jsonR = new File(Objects.requireNonNull(this.getClass().getClassLoader().getResource("io/a.json")).toURI()).toPath();
        jsonRM =
                new File(Objects.requireNonNull(this.getClass().getClassLoader().getResource("io/as.json")).toURI()).toPath();

        jsonX =
                new File(Objects.requireNonNull(this.getClass().getClassLoader().getResource("io/a.xml")).toURI()).toPath();
        jsonXM =
                new File(Objects.requireNonNull(this.getClass().getClassLoader().getResource("io/as.xml")).toURI()).toPath();
    }

    @AfterEach
    void tearDown() {
    }

    @Test
    void exportJSON() throws Exception {
        Path p = Files.createTempFile("temp", "test-frontend");
        EntityExporter.export(new Object[]{a}, FileType.JSON, p.toFile().getAbsolutePath());

        String serialization = Files.readString(p);
        String expectation = Files.readString(jsonR).replaceAll(" ", "").replaceAll("\r\n", "");
        assertEquals(serialization, expectation);
    }

    @Test
    void exportXML() throws Exception {
        Path p = Files.createTempFile("temp", "test-frontend");
        EntityExporter.export(new Object[]{a}, FileType.XML, p.toFile().getAbsolutePath());

        String serialization = Files.readString(p);
        String expectation = Files.readString(jsonX).replaceAll(" ", "").replaceAll("\r\n", "");
        assertEquals(serialization, expectation);
    }

    @Test
    void exportAsJSON1() throws Exception {
        Path p = Files.createTempFile("temp", "test-frontend");
        Assertions.assertDoesNotThrow(() -> EntityExporter.exportAsJSON(new Object[]{a}, p.toFile().getAbsolutePath()));
    }

    @Test
    void exportAsJSON2() throws Exception {
        Path p = Files.createTempFile("temp", "test-frontend");
        EntityExporter.exportAsJSON(new Object[]{a}, p.toFile().getAbsolutePath());
        String serialization = Files.readString(p);
        String expectation = Files.readString(jsonR).replaceAll(" ", "").replaceAll("\r\n", "");
        assertEquals(serialization, expectation);
    }

    @Test
    void exportAsXML1() throws Exception {
        Path p = Files.createTempFile("temp", "test-frontend");
        Assertions.assertDoesNotThrow(() -> EntityExporter.exportAsXML(new Object[]{a}, p.toFile().getAbsolutePath()));
    }

    @Test
    void exportAsXML2() throws Exception {
        Path p = Files.createTempFile("temp", "test-frontend");
        EntityExporter.exportAsXML(new Object[]{a}, p.toFile().getAbsolutePath());
        String serialization = Files.readString(p);
        String expectation = Files.readString(jsonX).replaceAll(" ", "").replaceAll("\r\n", "");
        assertEquals(serialization, expectation);
    }

    @Test
    void importToObjectsXMLA() throws Exception {
        A[] as = (A[]) EntityExporter.importToObjects(FileType.XML, jsonX.toAbsolutePath().toString(), A.class);
        Assertions.assertEquals(as[0], a);
    }

    @Test
    void importToObjectsJSONA() throws Exception {
        A[] as = (A[]) EntityExporter.importToObjects(FileType.JSON, jsonR.toAbsolutePath().toString(), A.class);
        Assertions.assertEquals(as[0], a);
    }

    @Test
    void importToObjectsFromXML1() {
        Assertions.assertDoesNotThrow(() ->
                (A[]) EntityExporter.importToObjectsFromXML(jsonX.toAbsolutePath().toString(), A.class));
    }

    @Test
    void importToObjectsFromXML2() throws Exception {
        A[] as = (A[]) EntityExporter.importToObjectsFromXML(jsonX.toAbsolutePath().toString(), A.class);
        Assertions.assertEquals(as[0], a);
    }

    @Test
    void importToObjectsFromXML3() throws Exception {
        A[] as = (A[]) EntityExporter.importToObjectsFromXML(jsonXM.toAbsolutePath().toString(), A.class);
        Assertions.assertEquals(as[0], a);
        Assertions.assertEquals(as[1], a);
    }

    @Test
    void importToObjectsFromJSON1() {
        Assertions.assertDoesNotThrow(() ->
                (A[]) EntityExporter.importToObjectsFromJSON(jsonR.toAbsolutePath().toString(), A.class));
    }

    @Test
    void importToObjectsFromJSON2() throws Exception {
        A[] as = (A[]) EntityExporter.importToObjectsFromJSON(jsonR.toAbsolutePath().toString(), A.class);
        Assertions.assertEquals(as[0], a);
    }

    @Test
    void importToObjectsFromJSON3() throws Exception {
        A[] as = (A[]) EntityExporter.importToObjectsFromJSON(jsonRM.toAbsolutePath().toString(), A.class);
        Assertions.assertEquals(as[0], a);
        Assertions.assertEquals(as[1], a);
    }

    @Test
    void importToObjectsFromJSON4() {
        Assertions.assertThrows(Exception.class,
                () -> EntityExporter.importToObjectsFromJSON("voidPath",
                        A.class));
    }
}