package cz.zcu.fav.kiv.mjakubas.saf.io.properties;

import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.net.URISyntaxException;
import java.util.Properties;

class PropertiesIOTest {

    @Test
    void getValue1() throws Exception {
        String path = new File(this.getClass().getClassLoader().getResource("testResources.properties").toURI()).getPath();
        Properties p = PropertiesIO.getProperties(path);
        String val = PropertiesIO.getValue(p, "one");
        Assertions.assertEquals("a", val);
    }

    @Test
    void getValue2() throws Exception {
        String path = new File(this.getClass().getClassLoader().getResource("testResources.properties").toURI()).getPath();
        Properties p = PropertiesIO.getProperties(path);
        Assertions.assertThrows(Exception.class, () -> PropertiesIO.getValue(p, "nonexistant"));
    }

    @Test
    void getFileTypeValue1() throws Exception {
        String path = new File(this.getClass().getClassLoader().getResource("testResources.properties").toURI()).getPath();
        Properties p = PropertiesIO.getProperties(path);
        FileType f = PropertiesIO.getFileTypeValue(p, "XML");
        Assertions.assertEquals(f, FileType.XML);
    }

    @Test
    void getFileTypeValue2() throws Exception {
        String path = new File(this.getClass().getClassLoader().getResource("testResources.properties").toURI()).getPath();
        Properties p = PropertiesIO.getProperties(path);
        FileType f = PropertiesIO.getFileTypeValue(p, "JSON");
        Assertions.assertEquals(f, FileType.JSON);
    }

    @Test
    void getFileTypeValueVoid() throws Exception {
        String path = new File(this.getClass().getClassLoader().getResource("testResources.properties").toURI()).getPath();
        Properties p = PropertiesIO.getProperties(path);
        Assertions.assertThrows(Exception.class, () -> PropertiesIO.getFileTypeValue(p, "A"));
    }

    @Test
    void getPropertiesExistent() throws URISyntaxException {
        String path = new File(this.getClass().getClassLoader().getResource("testResources.properties").toURI()).getPath();
        Assertions.assertDoesNotThrow(() -> PropertiesIO.getProperties(path));
    }

    @Test
    void getPropertiesNonExistent() {
        Assertions.assertThrows(Exception.class, () -> PropertiesIO.getProperties("wrongTestResources.properties"));
    }

    @Test
    void getPropertiesExistentContent() throws Exception {
        String path = new File(this.getClass().getClassLoader().getResource("testResources.properties").toURI()).getPath();
        Properties p = PropertiesIO.getProperties(path);
        Assertions.assertEquals("a", p.getProperty("one"));
    }
}