package cz.zcu.fav.kiv.mjakubas.saf.parser.mapper;

import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.exception.MapperException;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class JSONMapperTest {

    A testObject;

    @BeforeEach
    void setUp() {
        testObject = new A();
        testObject.setNumber(5);
        testObject.setText("a");

        B b = new B();
        b.setArray(new int[]{0, 1, 2, 3});
        testObject.setB(b);
    }

    @AfterEach
    void tearDown() {
    }

    @Test
    void testSerializeOneCorrect() throws MapperException {
        String serialization = new JSONMapper().serialize(testObject);
        Assertions.assertEquals(serialization, "{\"number\":5,\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}}");
    }

    @Test
    void testSerializeArrayCorrect() throws MapperException {
        String serialization = new JSONMapper().serialize(new Object[]{testObject, testObject});
        Assertions.assertEquals(serialization, "[{\"number\":5,\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}},{\"number\":5,\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}}]");
    }

    @Test
    void testSerializeMalformed() throws MapperException {
        String serialization = new JSONMapper().serialize(testObject);
        Assertions.assertNotEquals(serialization, "{\"numb\":5,\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}}");
    }

    @Test
    void testSerializeArrayMalformed() throws MapperException {
        String serialization = new JSONMapper().serialize(new Object[]{testObject, testObject});
        Assertions.assertNotEquals(serialization, "[{\"num\":5,\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}}," +
                "{\"number\":5,\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}}]");
    }

    @Test
    void testDeserializeOneCorrect() throws MapperException {
        String serialization = "{\"number\":5,\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}}";
        Assertions.assertEquals(testObject, new JSONMapper().deserializeOne(serialization, A.class));
    }

    @Test
    void testDeserializeArrayCorrect() throws MapperException {
        String serialization = "[{\"number\":5,\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}},{\"number\":5," +
                "\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}}]";
        A[] des = new JSONMapper().deserialize(serialization, A.class);
        for (A a : des) {
            Assertions.assertEquals(a, testObject);
        }
    }

    @Test
    void testDeserializeOneMalformed() throws MapperException {
        String serialization = "{\"number\":5,\"text\":\"a\",\"b\":{\"rray\":[0,1,2,3]}}";
        Assertions.assertThrows(MapperException.class, () -> new JSONMapper().deserializeOne(serialization, A.class));
    }

    @Test
    void testDeserializeArrayMalformed() throws MapperException {
        String serialization = "[{\"number\":5,\"text\":\"\",\"b\":{\"array\":[0,1,2,3]}},{\"number\":5," +
                "\"text\":\"a\",\"b\":{\"array\":[0,1,2,3]}}]";
        Assertions.assertThrows(MapperException.class, () -> new JSONMapper().deserializeOne(serialization, A.class));
    }
}