package cz.zcu.fav.kiv.mjakubas.saf.parser.mapper;

import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.exception.MapperException;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class XMLMapperTest {

    A testObject;

    @BeforeEach
    void setUp() {
        testObject = new A();
        testObject.setNumber(5);
        testObject.setText("a");

        B b = new B();
        b.setArray(new int[]{0, 1, 2, 3});
        testObject.setB(b);
    }

    @AfterEach
    void tearDown() {
    }

    @Test
    void testSerializeOneCorrect() throws MapperException {
        String serialization = new XMLMapper().serialize(testObject);
        Assertions.assertEquals(serialization, "<A><number>5</number><text>a</text><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></A>");
    }

    @Test
    void testSerializeArrayCorrect() throws MapperException {
        String serialization = new XMLMapper().serialize(new Object[]{testObject, testObject});
        Assertions.assertEquals(serialization, "<Objects><item><number>5</number><text>a</text><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></item><item><number>5</number><text>a</text><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></item></Objects>");
    }

    @Test
    void testSerializeMalformed() throws MapperException {
        String serialization = new XMLMapper().serialize(testObject);
        Assertions.assertNotEquals(serialization, "<A><numbers>5</number><text>a</text><b><array><array>0</array" +
                "><array>1</array><array>2</array><array>3</array></array></b></A>");
    }

    @Test
    void testSerializeArrayMalformed() throws MapperException {
        String serialization = new XMLMapper().serialize(new Object[]{testObject, testObject});
        Assertions.assertNotEquals(serialization, "<Objects><item><numbers>5</number><text>a</text><b><array><array>0" +
                "</array><array>1</array><array>2</array><array>3</array></array></b></item><item><number>5</number><text>a</text><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></item></Objects>");
    }

    @Test
    void testDeserializeOneCorrect() throws MapperException {
        String serialization = "<A><number>5</number><text>a</text><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></A>";
        Assertions.assertEquals(testObject, new XMLMapper().deserializeOne(serialization, A.class));
    }

    @Test
    void testDeserializeArrayCorrect() throws MapperException {
        String serialization = "<Objects><item><number>5</number><text>a</text><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></item><item><number>5</number><text>a</text><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></item></Objects>";
        A[] des = new XMLMapper().deserialize(serialization, A.class);
        for (A a : des) {
            Assertions.assertEquals(a, testObject);
        }
    }

    @Test
    void testDeserializeOneMalformed() throws MapperException {
        String serialization = "<A><number>5</number><text>a</t><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></A>";
        Assertions.assertThrows(MapperException.class, () -> new XMLMapper().deserializeOne(serialization, A.class));
    }

    @Test
    void testDeserializeArrayMalformed() throws MapperException {
        String serialization = "<Objects><item><number>5</number><text>a</text><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></item><item><number>5</number><text>axt><b><array><array>0</array><array>1</array><array>2</array><array>3</array></array></b></item></Objects>";
        Assertions.assertThrows(MapperException.class, () -> new XMLMapper().deserializeOne(serialization, A.class));
    }
}