package cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer;

import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer.exception.TokenException;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import static org.junit.jupiter.api.Assertions.*;

class TokenizerTest {

    Tokenizer tokenizer;

    public static final char C = Tokenizer.COMMAND_STRING_TOKEN_IDENTIFIER;
    public static final char V = Tokenizer.CLASS_TOKEN_IDENTIFIER;

    public static final char F = Tokenizer.FILETYPE_CLASS_TOKEN_IDENTIFIER;

    @BeforeEach
    void setUp() throws IOException {
        Properties properties = new Properties();
        properties.load(this.getClass().getClassLoader().getResourceAsStream("config.cmd.properties"));
        tokenizer = new Tokenizer(properties);
    }

    @AfterEach
    void tearDown() {
    }


    @Test
    void tokenizeSuccessful() throws TokenException {
        List<String> args = new ArrayList<>();
        args.add("{query.absences}");
        args.add("{httpMethod.get}");
        args.add("[query.sub.type]");
        args.add("<java.lang.String>");

        List<Token> tokenization = tokenizer.tokenize(args.toArray(new String[0]));

        Assertions.assertEquals("absences", tokenization.get(0).getStringToken());
        Assertions.assertEquals("--get", tokenization.get(1).getStringToken());
        Assertions.assertEquals(FileType.class, tokenization.get(2).getClassToken());
        Assertions.assertEquals(String.class, tokenization.get(3).getClassToken());
    }

    @Test
    void tokenizeUnSuccessful() throws TokenException {
        List<String> args = new ArrayList<>();
        args.add("{query.absences}");
        args.add("{query.absences}");
        args.add("[query.sub.type]");
        args.add("<java.lang.String>");

        List<Token> tokenization = tokenizer.tokenize(args.toArray(new String[0]));

        Assertions.assertEquals("absences", tokenization.get(0).getStringToken());
        Assertions.assertNotEquals("--get", tokenization.get(1).getStringToken());
        Assertions.assertEquals(FileType.class, tokenization.get(2).getClassToken());
        Assertions.assertEquals(String.class, tokenization.get(3).getClassToken());
    }

    @Test
    void tokenizeUnSuccessful1() throws TokenException {
        List<String> args = new ArrayList<>();
        args.add("{query.absences}");
        args.add("{query.absences}");
        args.add("[query.sub.type]");
        args.add("<java.lang.Strng>");

        Assertions.assertThrows(TokenException.class, () -> tokenizer.tokenize(args.toArray(new String[0])));
    }


    @Test
    void isClassValueTrue() {
        String someString = V + "someString";
        Assertions.assertTrue(tokenizer.isClassValue(someString));
    }

    @Test
    void isClassValueFalse() {
        String someString = "someString";
        Assertions.assertFalse(tokenizer.isClassValue(someString));
    }

    @Test
    void isFileTypeValueTrue() {
        String someString = F + "someString";
        Assertions.assertTrue(tokenizer.isFileTypeValue(someString));
    }

    @Test
    void isFileTypeValueFalse() {
        String someString = "someString";
        Assertions.assertFalse(tokenizer.isFileTypeValue(someString));
    }

    @Test
    void processClassValueInteger() throws ClassNotFoundException {
        String classValue = V + "java.lang.Integer" + V;
        Class<?> c = tokenizer.processClassValue(classValue);
        Assertions.assertEquals(c, Integer.class);
    }

    @Test
    void processClassValueString() throws ClassNotFoundException {
        String classValue = V + "java.lang.String" + V;
        Class<?> c = tokenizer.processClassValue(classValue);
        Assertions.assertEquals(c, String.class);
    }

    @Test
    void processClassValueMalformed1() {
        String classValue = V + "java.lang.Strin" + V;
        Assertions.assertThrows(ClassNotFoundException.class, () -> tokenizer.processClassValue(classValue));
    }

    @Test
    void processClassValueMalformed2() {
        String classValue = V + "String" + V;
        Assertions.assertThrows(ClassNotFoundException.class, () -> tokenizer.processClassValue(classValue));
    }

    @Test
    void testProcessFileTypeValue() {
        Assertions.assertEquals(FileType.class, tokenizer.processFileTypeValue());
    }

    @Test
    void testProcessStringValue() throws IOException {
        String value = tokenizer.processStringValue("string");
        assertEquals(value, "string");
    }

    @Test
    void testProcessStringValueP1() throws IOException {
        String value = tokenizer.processStringValue(C + "query.absences" + C);
        assertEquals(value, "absences");
    }

    @Test
    void testProcessStringValueMalformed() {
        assertThrows(IOException.class, () -> tokenizer.processStringValue(C + "query.absences"));
    }
}